# Copyright 2014 Hewlett-Packard Development Company, L.P.
#
# Author: Endre Karlson <endre.karlson@hp.com>
#
# Licensed under the Apache License, Version 2.0 (the "License"); you may
# not use this file except in compliance with the License. You may obtain
# a copy of the License at
#
#      http://www.apache.org/licenses/LICENSE-2.0
#
# Unless required by applicable law or agreed to in writing, software
# distributed under the License is distributed on an "AS IS" BASIS, WITHOUT
# WARRANTIES OR CONDITIONS OF ANY KIND, either express or implied. See the
# License for the specific language governing permissions and limitations
# under the License.

import ddt
import mock

from rally.plugins.openstack.scenarios.designate import utils
from tests.unit import test

DESIGNATE_UTILS = "rally.plugins.openstack.scenarios.designate.utils."


@ddt.ddt
class DesignateScenarioTestCase(test.ScenarioTestCase):

    def setUp(self):
        super(DesignateScenarioTestCase, self).setUp()
        self.domain = mock.Mock()
        self.zone = mock.Mock()
        self.server = mock.Mock()

        self.client = self.clients("designate", version="2")

    @ddt.data(
        {},
        {"email": "root@zone.name"})
    def test_create_domain(self, domain_data):
        random_name = "foo"
        scenario = utils.DesignateScenario(context=self.context)
        scenario.generate_random_name = mock.Mock(return_value=random_name)
        self.clients("designate").domains.create.return_value = self.domain
        expected = {"email": "root@random.name"}
        expected.update(domain_data)
        expected["name"] = "%s.name." % random_name

        domain = scenario._create_domain(domain_data)
        self.clients("designate").domains.create.assert_called_once_with(
            expected)
        self.assertEqual(self.domain, domain)
        self._test_atomic_action_timer(scenario.atomic_actions(),
                                       "designate.create_domain")

    def test_list_domains(self):
        scenario = utils.DesignateScenario(context=self.context)
        return_domains_list = scenario._list_domains()
        self.assertEqual(self.clients("designate").domains.list.return_value,
                         return_domains_list)
        self._test_atomic_action_timer(scenario.atomic_actions(),
                                       "designate.list_domains")

    def test_delete_domain(self):
        scenario = utils.DesignateScenario(context=self.context)

        domain = scenario._create_domain()
        scenario._delete_domain(domain["id"])
        self._test_atomic_action_timer(scenario.atomic_actions(),
                                       "designate.delete_domain")

    def test_update_domain(self):
        scenario = utils.DesignateScenario(context=self.context)
        domain = scenario._create_domain()
        self.clients("designate").domains.update.return_value = self.domain
        updated_domain = scenario._update_domain(domain)
        self.clients("designate").domains.update.assert_called_once_with(
            domain)
        self.assertEqual(self.domain, updated_domain)
        self._test_atomic_action_timer(scenario.atomic_actions(),
                                       "designate.update_domain")

    @ddt.data(
        {},
        {"data": "127.0.0.1"})
    def test_create_record(self, record_data):
        random_name = "foo"
        domain_name = "zone.name."
        domain = {"name": domain_name, "id": "123"}
        record_name = "%s.%s" % (random_name, domain_name)

        scenario = utils.DesignateScenario(context=self.context)
        scenario.generate_random_name = mock.Mock(return_value=random_name)

        expected = {"type": "A", "data": "10.0.0.1"}
        expected.update(record_data)
        expected["name"] = record_name

        scenario._create_record(domain, record=record_data)
        self.clients("designate").records.create.assert_called_once_with(
            domain["id"], expected)
        self._test_atomic_action_timer(scenario.atomic_actions(),
                                       "designate.create_record")

    def test_list_records(self):
        scenario = utils.DesignateScenario(context=self.context)
        return_records_list = scenario._list_records("123")
        self.assertEqual(self.clients("designate").records.list.return_value,
                         return_records_list)
        self._test_atomic_action_timer(scenario.atomic_actions(),
                                       "designate.list_records")

    def test_delete_record(self):
        scenario = utils.DesignateScenario(context=self.context)

        domain_id = mock.Mock()
        record_id = mock.Mock()
        scenario._delete_record(domain_id, record_id)
        self.clients("designate").records.delete.assert_called_once_with(
            domain_id, record_id)
        self._test_atomic_action_timer(scenario.atomic_actions(),
                                       "designate.delete_record")

        self.clients("designate").records.delete.reset_mock()
        scenario._delete_record(domain_id, record_id, atomic_action=False)
        self.clients("designate").records.delete.assert_called_once_with(
            domain_id, record_id)

    def test_create_server(self):
        scenario = utils.DesignateScenario(context=self.context)
        random_name = "foo"
        scenario.generate_random_name = mock.Mock(return_value=random_name)

        explicit_name = "bar.io."

        self.admin_clients(
            "designate").servers.create.return_value = self.server

        # Check that the defaults / randoms are used if nothing is specified
        server = scenario._create_server()
        self.admin_clients("designate").servers.create.assert_called_once_with(
            {"name": "name.%s." % random_name})
        self.assertEqual(self.server, server)
        self._test_atomic_action_timer(scenario.atomic_actions(),
                                       "designate.create_server")

        self.admin_clients("designate").servers.create.reset_mock()

        # Check that when specifying server name defaults are not used...
        data = {"name": explicit_name}
        server = scenario._create_server(data)
        self.admin_clients(
            "designate").servers.create.assert_called_once_with(data)
        self.assertEqual(self.server, server)

    def test_delete_server(self):
        scenario = utils.DesignateScenario(context=self.context)

        scenario._delete_server("foo_id")
        self.admin_clients("designate").servers.delete.assert_called_once_with(
            "foo_id")
        self._test_atomic_action_timer(scenario.atomic_actions(),
                                       "designate.delete_server")

    # NOTE: API V2
    @ddt.data(
        {},
        {"email": "root@zone.name"},
        {"name": "example.name."},
        {
            "email": "root@zone.name",
            "name": "example.name."
        })
    def test_create_zone(self, zone_data):
        scenario = utils.DesignateScenario()

        random_name = "foo"

        scenario = utils.DesignateScenario(context=self.context)
        scenario.generate_random_name = mock.Mock(return_value=random_name)
        self.client.zones.create.return_value = self.zone

        expected = {
            "email": "root@random.name",
            "name": "%s.name." % random_name,
            "type_": "PRIMARY"
        }
        expected.update(zone_data)

        # Check that the defaults / randoms are used if nothing is specified
        zone = scenario._create_zone(**zone_data)
        self.client.zones.create.assert_called_once_with(
            description=None,
            ttl=None,
            **expected)
        self.assertEqual(self.zone, zone)
        self._test_atomic_action_timer(scenario.atomic_actions(),
                                       "designate.create_zone")

    def test_list_zones(self):
        scenario = utils.DesignateScenario(context=self.context)
        return_zones_list = scenario._list_zones()
        self.assertEqual(self.client.zones.list.return_value,
                         return_zones_list)
        self._test_atomic_action_timer(scenario.atomic_actions(),
                                       "designate.list_zones")

    def test_delete_zone(self):
        scenario = utils.DesignateScenario(context=self.context)

        zone = scenario._create_zone()
        scenario._delete_zone(zone["id"])
        self._test_atomic_action_timer(scenario.atomic_actions(),
                                       "designate.delete_zone")

    def test_list_recordsets(self):
        scenario = utils.DesignateScenario(context=self.context)
        return_recordsets_list = scenario._list_recordsets("123")
        self.assertEqual(
            self.client.recordsets.list.return_value,
            return_recordsets_list)
        self._test_atomic_action_timer(scenario.atomic_actions(),
                                       "designate.list_recordsets")



# Copyright 2013: Mirantis Inc.
# All Rights Reserved.
#
#    Licensed under the Apache License, Version 2.0 (the "License"); you may
#    not use this file except in compliance with the License. You may obtain
#    a copy of the License at
#
#         http://www.apache.org/licenses/LICENSE-2.0
#
#    Unless required by applicable law or agreed to in writing, software
#    distributed under the License is distributed on an "AS IS" BASIS, WITHOUT
#    WARRANTIES OR CONDITIONS OF ANY KIND, either express or implied. See the
#    License for the specific language governing permissions and limitations
#    under the License.

"""Tests for db.deploy layer."""


import mock

from rally.common import objects
from rally import consts
from tests.unit import test


class DeploymentTestCase(test.TestCase):
    def setUp(self):
        super(DeploymentTestCase, self).setUp()
        self.deployment = {
            "uuid": "baa1bfb6-0c38-4f6c-9bd0-45968890e4f4",
            "name": "",
            "config": {},
            "endpoint": {},
            "status": consts.DeployStatus.DEPLOY_INIT,
        }
        self.resource = {
            "id": 42,
            "deployment_uuid": self.deployment["uuid"],
            "provider_name": "provider",
            "type": "some",
            "info": {"key": "value"},
        }

    @mock.patch("rally.common.objects.deploy.db.deployment_create")
    def test_init_with_create(self, mock_deployment_create):
        mock_deployment_create.return_value = self.deployment
        deploy = objects.Deployment()
        mock_deployment_create.assert_called_once_with({})
        self.assertEqual(deploy["uuid"], self.deployment["uuid"])

    @mock.patch("rally.common.objects.deploy.db.deployment_create")
    def test_init_without_create(self, mock_deployment_create):
        deploy = objects.Deployment(deployment=self.deployment)
        self.assertFalse(mock_deployment_create.called)
        self.assertEqual(deploy["uuid"], self.deployment["uuid"])

    @mock.patch("rally.common.objects.deploy.db.deployment_get")
    def test_get(self, mock_deployment_get):
        mock_deployment_get.return_value = self.deployment
        deploy = objects.Deployment.get(self.deployment["uuid"])
        mock_deployment_get.assert_called_once_with(self.deployment["uuid"])
        self.assertEqual(deploy["uuid"], self.deployment["uuid"])

    @mock.patch("rally.common.objects.deploy.db.deployment_delete")
    @mock.patch("rally.common.objects.deploy.db.deployment_create")
    def test_create_and_delete(self, mock_deployment_create,
                               mock_deployment_delete):
        mock_deployment_create.return_value = self.deployment
        deploy = objects.Deployment()
        deploy.delete()
        mock_deployment_delete.assert_called_once_with(self.deployment["uuid"])

    @mock.patch("rally.common.objects.deploy.db.deployment_delete")
    def test_delete_by_uuid(self, mock_deployment_delete):
        objects.Deployment.delete_by_uuid(self.deployment["uuid"])
        mock_deployment_delete.assert_called_once_with(self.deployment["uuid"])

    @mock.patch("rally.common.objects.deploy.db.deployment_update")
    @mock.patch("rally.common.objects.deploy.db.deployment_create")
    def test_update(self, mock_deployment_create, mock_deployment_update):
        mock_deployment_create.return_value = self.deployment
        mock_deployment_update.return_value = {"opt": "val2"}
        deploy = objects.Deployment(opt="val1")
        deploy._update({"opt": "val2"})
        mock_deployment_update.assert_called_once_with(
            self.deployment["uuid"], {"opt": "val2"})
        self.assertEqual(deploy["opt"], "val2")

    @mock.patch("rally.common.objects.deploy.db.deployment_update")
    def test_update_status(self, mock_deployment_update):
        mock_deployment_update.return_value = self.deployment
        deploy = objects.Deployment(deployment=self.deployment)
        deploy.update_status(consts.DeployStatus.DEPLOY_FAILED)
        mock_deployment_update.assert_called_once_with(
            self.deployment["uuid"],
            {"status": consts.DeployStatus.DEPLOY_FAILED},
        )

    @mock.patch("rally.common.objects.deploy.db.deployment_update")
    def test_update_name(self, mock_deployment_update):
        mock_deployment_update.return_value = self.deployment
        deploy = objects.Deployment(deployment=self.deployment)
        deploy.update_name("new_name")
        mock_deployment_update.assert_called_once_with(
            self.deployment["uuid"],
            {"name": "new_name"},
        )

    @mock.patch("rally.common.objects.deploy.db.deployment_update")
    def test_update_config(self, mock_deployment_update):
        mock_deployment_update.return_value = self.deployment
        deploy = objects.Deployment(deployment=self.deployment)
        deploy.update_config({"opt": "val"})
        mock_deployment_update.assert_called_once_with(
            self.deployment["uuid"],
            {"config": {"opt": "val"}},
        )

    @mock.patch("rally.common.objects.deploy.db.deployment_update")
    def test_update_credentials(self, mock_deployment_update):
        mock_deployment_update.return_value = self.deployment
        deploy = objects.Deployment(deployment=self.deployment)
        credentials = {
            "admin": objects.Credential("url", "user", "pwd", "tenant",
                                        consts.EndpointPermission.ADMIN),
            "users": [
                objects.Credential("url1", "user1", "pwd1", "tenant1",
                                   consts.EndpointPermission.USER),
                objects.Credential("url2", "user2", "pwd2", "tenant2",
                                   consts.EndpointPermission.USER),
            ]
        }

        expected_users = [u.to_dict(include_permission=True)
                          for u in credentials["users"]]

        deploy.update_credentials(credentials)
        mock_deployment_update.assert_called_once_with(
            self.deployment["uuid"],
            {
                "admin": credentials["admin"].to_dict(include_permission=True),
                "users": expected_users
            })

    @mock.patch("rally.common.objects.deploy.db.deployment_update")
    def test_update_empty_credentials(self, mock_deployment_update):
        mock_deployment_update.return_value = self.deployment
        deploy = objects.Deployment(deployment=self.deployment)
        deploy.update_credentials({})
        mock_deployment_update.assert_called_once_with(
            self.deployment["uuid"], {"admin": {}, "users": []})

    @mock.patch("rally.common.objects.deploy.db.resource_create")
    def test_add_resource(self, mock_resource_create):
        mock_resource_create.return_value = self.resource
        deploy = objects.Deployment(deployment=self.deployment)
        resource = deploy.add_resource("provider", type="some",
                                       info={"key": "value"})
        self.assertEqual(resource["id"], self.resource["id"])
        mock_resource_create.assert_called_once_with({
            "deployment_uuid": self.deployment["uuid"],
            "provider_name": "provider",
            "type": "some",
            "info": {"key": "value"},
        })

    @mock.patch("rally.common.objects.task.db.resource_delete")
    def test_delete(self, mock_resource_delete):
        objects.Deployment.delete_resource(42)
        mock_resource_delete.assert_called_once_with(42)

    @mock.patch("rally.common.objects.task.db.resource_get_all")
    def test_get_resources(self, mock_resource_get_all):
        mock_resource_get_all.return_value = [self.resource]
        deploy = objects.Deployment(deployment=self.deployment)
        resources = deploy.get_resources(provider_name="provider", type="some")
        self.assertEqual(len(resources), 1)
        self.assertEqual(resources[0]["id"], self.resource["id"])

    @mock.patch("rally.common.objects.deploy.datetime.datetime")
    @mock.patch("rally.common.objects.deploy.db.deployment_update")
    def test_update_set_started(self, mock_deployment_update, mock_datetime):
        mock_datetime.now = mock.Mock(return_value="fake_time")
        mock_deployment_update.return_value = self.deployment
        deploy = objects.Deployment(deployment=self.deployment)
        deploy.set_started()
        mock_deployment_update.assert_called_once_with(
            self.deployment["uuid"],
            {"started_at": "fake_time",
             "status": consts.DeployStatus.DEPLOY_STARTED}
        )

    @mock.patch("rally.common.objects.deploy.datetime.datetime")
    @mock.patch("rally.common.objects.deploy.db.deployment_update")
    def test_update_set_completed(self, mock_deployment_update, mock_datetime):
        mock_datetime.now = mock.Mock(return_value="fake_time")
        mock_deployment_update.return_value = self.deployment
        deploy = objects.Deployment(deployment=self.deployment)
        deploy.set_completed()
        mock_deployment_update.assert_called_once_with(
            self.deployment["uuid"],
            {"completed_at": "fake_time",
             "status": consts.DeployStatus.DEPLOY_FINISHED}
        )



# Copyright 2013: Mirantis Inc.
# All Rights Reserved.
#
#    Licensed under the Apache License, Version 2.0 (the "License"); you may
#    not use this file except in compliance with the License. You may obtain
#    a copy of the License at
#
#         http://www.apache.org/licenses/LICENSE-2.0
#
#    Unless required by applicable law or agreed to in writing, software
#    distributed under the License is distributed on an "AS IS" BASIS, WITHOUT
#    WARRANTIES OR CONDITIONS OF ANY KIND, either express or implied. See the
#    License for the specific language governing permissions and limitations
#    under the License.

""" Rally command: deployment """

from __future__ import print_function

import json
import os
import re
import sys

import jsonschema
from keystoneclient import exceptions as keystone_exceptions
from six.moves.urllib import parse
import yaml

from rally import api
from rally.cli import cliutils
from rally.cli import envutils
from rally.common import fileutils
from rally.common.i18n import _
from rally.common import utils
from rally import exceptions
from rally import plugins


class DeploymentCommands(object):
    """Set of commands that allow you to manage deployments."""

    @cliutils.args("--name", type=str, required=True,
                   help="A name of the deployment.")
    @cliutils.args("--fromenv", action="store_true",
                   help="Read environment variables instead of config file.")
    @cliutils.args("--filename", type=str, required=False, metavar="<path>",
                   help="A path to the configuration file of the "
                   "deployment.")
    @cliutils.args("--no-use", action="store_false", dest="do_use",
                   help="Don\'t set new deployment as default for"
                        " future operations.")
    @plugins.ensure_plugins_are_loaded
    def create(self, name, fromenv=False, filename=None, do_use=False):
        """Create new deployment.

        This command will create new deployment record in rally database.
        In case of ExistingCloud deployment engine it will use cloud,
        represented in config.
        In cases when cloud doesn't exists Rally will deploy new one
        for you with Devstack or Fuel. For this purposes different deployment
        engines are developed.

        If you use ExistingCloud deployment engine you can pass deployment
        config by environment variables:
            OS_USERNAME
            OS_PASSWORD
            OS_AUTH_URL
            OS_TENANT_NAME
            OS_ENDPOINT
            OS_REGION_NAME
            OS_CACERT
            OS_INSECURE

        All other deployment engines need more complex configuration data, so
        it should be stored in configuration file.

        You can use physical servers, lxc containers, KVM virtual machines
        or virtual machines in OpenStack for deploying the cloud in.
        Except physical servers, Rally can create cluster nodes for you.
        Interaction with virtualization software, OpenStack
        cloud or physical servers is provided by server providers.

        :param fromenv: boolean, read environment instead of config file
        :param filename: a path to the configuration file
        :param name: a name of the deployment
        """

        if fromenv:
            config = {"type": "ExistingCloud"}
            config.update(envutils.get_creds_from_env_vars())
        else:
            if not filename:
                print("Either --filename or --fromenv is required.")
                return(1)
            filename = os.path.expanduser(filename)
            with open(filename, "rb") as deploy_file:
                config = yaml.safe_load(deploy_file.read())

        try:
            deployment = api.Deployment.create(config, name)
        except jsonschema.ValidationError:
            print(_("Config schema validation error: %s.") % sys.exc_info()[1])
            return(1)
        except exceptions.DeploymentNameExists:
            print(_("Error: %s") % sys.exc_info()[1])
            return(1)

        self.list(deployment_list=[deployment])
        if do_use:
            self.use(deployment["uuid"])

    @cliutils.args("--deployment", dest="deployment", type=str,
                   metavar="<uuid>", required=False,
                   help="UUID or name of a deployment.")
    @envutils.with_default_deployment()
    @plugins.ensure_plugins_are_loaded
    def recreate(self, deployment=None):
        """Destroy and create an existing deployment.

        Unlike 'deployment destroy' command deployment database record will
        not be deleted, so deployment's UUID stay same.

        :param deployment: a UUID or name of the deployment
        """
        api.Deployment.recreate(deployment)

    @cliutils.args("--deployment", dest="deployment", type=str,
                   metavar="<uuid>", required=False,
                   help="UUID or name of a deployment.")
    @envutils.with_default_deployment()
    @plugins.ensure_plugins_are_loaded
    def destroy(self, deployment=None):
        """Destroy existing deployment.

        This will delete all containers, virtual machines, OpenStack instances
        or Fuel clusters created during Rally deployment creation. Also it will
        remove deployment record from Rally database.

        :param deployment: a UUID or name of the deployment
        """
        api.Deployment.destroy(deployment)

    def list(self, deployment_list=None):
        """List existing deployments."""

        headers = ["uuid", "created_at", "name", "status", "active"]
        current_deployment = envutils.get_global("RALLY_DEPLOYMENT")
        deployment_list = deployment_list or api.Deployment.list()

        table_rows = []
        if deployment_list:
            for t in deployment_list:
                r = [str(t[column]) for column in headers[:-1]]
                r.append("" if t["uuid"] != current_deployment else "*")
                table_rows.append(utils.Struct(**dict(zip(headers, r))))
            cliutils.print_list(table_rows, headers,
                                sortby_index=headers.index("created_at"))
        else:
            print(_("There are no deployments. "
                    "To create a new deployment, use:"
                    "\nrally deployment create"))

    @cliutils.args("--deployment", dest="deployment", type=str,
                   metavar="<uuid>", required=False,
                   help="UUID or name of a deployment.")
    @envutils.with_default_deployment()
    @cliutils.suppress_warnings
    def config(self, deployment=None):
        """Display configuration of the deployment.

        Output is the configuration of the deployment in a
        pretty-printed JSON format.

        :param deployment: a UUID or name of the deployment
        """
        deploy = api.Deployment.get(deployment)
        result = deploy["config"]
        print(json.dumps(result, sort_keys=True, indent=4))

    @cliutils.args("--deployment", dest="deployment", type=str,
                   metavar="<uuid>", required=False,
                   help="UUID or name of a deployment.")
    @envutils.with_default_deployment()
    def show(self, deployment=None):
        """Show the credentials of the deployment.

        :param deployment: a UUID or name of the deployment
        """

        headers = ["auth_url", "username", "password", "tenant_name",
                   "region_name", "endpoint_type"]
        table_rows = []

        deployment = api.Deployment.get(deployment)
        users = deployment["users"]
        admin = deployment["admin"]
        credentials = users + [admin] if admin else users

        for ep in credentials:
            data = ["***" if m == "password" else ep.get(m, "")
                    for m in headers]
            table_rows.append(utils.Struct(**dict(zip(headers, data))))
        cliutils.print_list(table_rows, headers)

    @cliutils.args("--deployment", dest="deployment", type=str,
                   metavar="<uuid>", required=False,
                   help="UUID or name of a deployment.")
    @envutils.with_default_deployment()
    def check(self, deployment=None):
        """Check keystone authentication and list all available services.

        :param deployment: a UUID or name of the deployment
        """
        headers = ["services", "type", "status"]
        table_rows = []
        try:
            deployment = api.Deployment.get(deployment)

        except exceptions.DeploymentNotFound:
            print(_("Deployment %s is not found.") % deployment)
            return(1)

        try:
            services = api.Deployment.check(deployment)
        except keystone_exceptions.ConnectionRefused:
            print(_("Unable to connect %s.") % deployment["admin"]["auth_url"])
            return(1)

        except exceptions.InvalidArgumentsException:
            data = ["keystone", "identity", "Error"]
            table_rows.append(utils.Struct(**dict(zip(headers, data))))
            print(_("Authentication Issues: %s.")
                  % sys.exc_info()[1])
            return(1)

        for serv_type, serv in services.items():
            data = [serv, serv_type, "Available"]
            table_rows.append(utils.Struct(**dict(zip(headers, data))))
        print(_("keystone endpoints are valid and following"
              " services are available:"))
        cliutils.print_list(table_rows, headers)

    def _update_openrc_deployment_file(self, deployment, credential):
        openrc_path = os.path.expanduser("~/.rally/openrc-%s" % deployment)
        with open(openrc_path, "w+") as env_file:
            env_file.write("export OS_AUTH_URL=%(auth_url)s\n"
                           "export OS_USERNAME=%(username)s\n"
                           "export OS_PASSWORD=%(password)s\n"
                           "export OS_TENANT_NAME=%(tenant_name)s\n"
                           % credential)
            if credential.get("region_name"):
                env_file.write("export OS_REGION_NAME=%s\n" %
                               credential["region_name"])
            if credential.get("endpoint"):
                env_file.write("export OS_ENDPOINT=%s\n" %
                               credential["endpoint"])
            if credential.get("https_cacert"):
                env_file.write("export OS_CACERT=%s\n" %
                               credential["https_cacert"])
            if re.match(r"^/v3/?$", parse.urlparse(
                    credential["auth_url"]).path) is not None:
                env_file.write("export OS_USER_DOMAIN_NAME=%s\n"
                               "export OS_PROJECT_DOMAIN_NAME=%s\n" %
                               (credential["user_domain_name"],
                                credential["project_domain_name"]))
        expanded_path = os.path.expanduser("~/.rally/openrc")
        if os.path.exists(expanded_path):
            os.remove(expanded_path)
        os.symlink(openrc_path, expanded_path)

    @cliutils.args("--deployment", dest="deployment", type=str,
                   metavar="<uuid>", required=False,
                   help="UUID or name of a deployment.")
    def use(self, deployment):
        """Set active deployment.

        :param deployment: UUID or name of a deployment
        """
        try:
            deployment = api.Deployment.get(deployment)
            print("Using deployment: %s" % deployment["uuid"])

            fileutils.update_globals_file("RALLY_DEPLOYMENT",
                                          deployment["uuid"])
            self._update_openrc_deployment_file(
                deployment["uuid"],
                deployment["admin"] or deployment["users"][0])
            print ("~/.rally/openrc was updated\n\nHINTS:\n"
                   "* To get your cloud resources, run:\n\t"
                   "rally show [flavors|images|keypairs|networks|secgroups]\n"
                   "\n* To use standard OpenStack clients, set up your env by "
                   "running:\n\tsource ~/.rally/openrc\n"
                   "  OpenStack clients are now configured, e.g run:\n\t"
                   "glance image-list")
        except exceptions.DeploymentNotFound:
            print("Deployment %s is not found." % deployment)
            return 1



# Copyright 2015: Cisco Systems, Inc.
# All Rights Reserved.
#
#    Licensed under the Apache License, Version 2.0 (the "License"); you may
#    not use this file except in compliance with the License. You may obtain
#    a copy of the License at
#
#         http://www.apache.org/licenses/LICENSE-2.0
#
#    Unless required by applicable law or agreed to in writing, software
#    distributed under the License is distributed on an "AS IS" BASIS, WITHOUT
#    WARRANTIES OR CONDITIONS OF ANY KIND, either express or implied. See the
#    License for the specific language governing permissions and limitations
#    under the License.

from rally.plugins.openstack import scenario
from rally.task import atomic


class SwiftScenario(scenario.OpenStackScenario):
    """Base class for Swift scenarios with basic atomic actions."""

    @atomic.action_timer("swift.list_containers")
    def _list_containers(self, full_listing=True, **kwargs):
        """Return list of containers.

        :param full_listing: bool, enable unlimit number of listing returned
        :param kwargs: dict, other optional parameters to get_account

        :returns: tuple, (dict of response headers, a list of containers)
        """
        return self.clients("swift").get_account(full_listing=full_listing,
                                                 **kwargs)

    @atomic.optional_action_timer("swift.create_container")
    def _create_container(self, public=False, **kwargs):
        """Create a new container.

        :param public: bool, set container as public
        :param atomic_action: bool, enable create container to be
                              tracked as an atomic action. added and
                              handled by the optional_action_timer()
                              decorator
        :param kwargs: dict, other optional parameters to put_container

        :returns: container name
        """
        if public:
            kwargs.setdefault("headers", {})
            kwargs["headers"].setdefault("X-Container-Read", ".r:*,.rlistings")

        container_name = self.generate_random_name()

        self.clients("swift").put_container(container_name, **kwargs)
        return container_name

    @atomic.optional_action_timer("swift.delete_container")
    def _delete_container(self, container_name, **kwargs):
        """Delete a container with given name.

        :param container_name: str, name of the container to delete
        :param atomic_action: bool, enable delete container to be
                              tracked as an atomic action. added and
                              handled by the optional_action_timer()
                              decorator
        :param kwargs: dict, other optional parameters to delete_container
        """
        self.clients("swift").delete_container(container_name, **kwargs)

    @atomic.optional_action_timer("swift.list_objects")
    def _list_objects(self, container_name, full_listing=True, **kwargs):
        """Return objects inside container.

        :param container_name: str, name of the container to make the list
                               objects operation against
        :param full_listing: bool, enable unlimit number of listing returned
        :param atomic_action: bool, enable list objects to be tracked
                              as an atomic action. added and handled
                              by the optional_action_timer() decorator
        :param kwargs: dict, other optional parameters to get_container

        :returns: tuple, (dict of response headers, a list of objects)
        """
        return self.clients("swift").get_container(container_name,
                                                   full_listing=full_listing,
                                                   **kwargs)

    @atomic.optional_action_timer("swift.upload_object")
    def _upload_object(self, container_name, content, **kwargs):
        """Upload content to a given container.

        :param container_name: str, name of the container to upload object to
        :param content: file stream, content to upload
        :param atomic_action: bool, enable upload object to be tracked
                              as an atomic action. added and handled
                              by the optional_action_timer() decorator
        :param kwargs: dict, other optional parameters to put_object

        :returns: tuple, (etag and object name)
        """
        object_name = self.generate_random_name()

        return (self.clients("swift").put_object(container_name, object_name,
                                                 content, **kwargs),
                object_name)

    @atomic.optional_action_timer("swift.download_object")
    def _download_object(self, container_name, object_name, **kwargs):
        """Download object from container.

        :param container_name: str, name of the container to download object
                               from
        :param object_name: str, name of the object to download
        :param atomic_action: bool, enable download object to be
                              tracked as an atomic action. added and
                              handled by the optional_action_timer()
                              decorator
        :param kwargs: dict, other optional parameters to get_object

        :returns: tuple, (dict of response headers, the object's contents)
        """
        return self.clients("swift").get_object(container_name, object_name,
                                                **kwargs)

    @atomic.optional_action_timer("swift.delete_object")
    def _delete_object(self, container_name, object_name, **kwargs):
        """Delete object from container.

        :param container_name: str, name of the container to delete object from
        :param object_name: str, name of the object to delete
        :param atomic_action: bool, enable delete object to be tracked
                              as an atomic action. added and handled
                              by the optional_action_timer() decorator
        :param kwargs: dict, other optional parameters to delete_object
        """
        self.clients("swift").delete_object(container_name, object_name,
                                            **kwargs)



# Copyright 2014: Mirantis Inc.
# All Rights Reserved.
#
#    Licensed under the Apache License, Version 2.0 (the "License"); you may
#    not use this file except in compliance with the License. You may obtain
#    a copy of the License at
#
#         http://www.apache.org/licenses/LICENSE-2.0
#
#    Unless required by applicable law or agreed to in writing, software
#    distributed under the License is distributed on an "AS IS" BASIS, WITHOUT
#    WARRANTIES OR CONDITIONS OF ANY KIND, either express or implied. See the
#    License for the specific language governing permissions and limitations
#    under the License.

import ddt

from rally import exceptions
from rally.task.processing import utils
from tests.unit import test


class MathTestCase(test.TestCase):

    def test_percentile(self):
        lst = list(range(1, 101))
        result = utils.percentile(lst, 0.1)
        self.assertEqual(result, 10.9)

    def test_percentile_value_none(self):
        result = utils.percentile(None, 0.1)
        self.assertIsNone(result)

    def test_percentile_equal(self):
        lst = list(range(1, 101))
        result = utils.percentile(lst, 1)
        self.assertEqual(result, 100)

    def test_mean(self):
        lst = list(range(1, 100))
        result = utils.mean(lst)
        self.assertEqual(result, 50.0)

    def test_mean_empty_list(self):
        lst = []
        self.assertRaises(exceptions.InvalidArgumentsException,
                          utils.mean, lst)

    def test_median_single_value(self):
        lst = [5]
        result = utils.median(lst)
        self.assertEqual(5, result)

    def test_median_odd_sized_list(self):
        lst = [1, 2, 3, 4, 5]
        result = utils.median(lst)
        self.assertEqual(3, result)

    def test_median_even_sized_list(self):
        lst = [1, 2, 3, 4]
        result = utils.median(lst)
        self.assertEqual(2.5, result)

    def test_median_empty_list(self):
        lst = []
        self.assertRaises(ValueError,
                          utils.median, lst)

        lst = None
        self.assertRaises(ValueError,
                          utils.median, lst)


@ddt.ddt
class GraphZipperTestCase(test.TestCase):

    @ddt.data({"data_stream": list(range(1, 11)), "zipped_size": 8,
               "expected": [[1, 1.2], [3, 2.4], [4, 3.6], [5, 4.8], [7, 6.2],
                            [8, 7.4], [9, 8.6], [10, 9.8]]},
              {"data_stream": [.005, .8, 22, .004, .7, 12, .5, .07, .02] * 10,
               "zipped_size": 8, "expected": [
                   [1, 3.769244444444445], [18, 4.706933333333334],
                   [29, 4.339911111111111], [40, 3.2279111111111116],
                   [52, 3.769244444444445], [63, 4.706933333333334],
                   [74, 4.339911111111111], [90, 3.2279111111111116]]},
              {"data_stream": list(range(1, 100)), "zipped_size": 1000,
               "expected": [[i, i] for i in range(1, 100)]},
              {"data_stream": [1, 4, 11, None, 42], "zipped_size": 1000,
               "expected": [[1, 1], [2, 4], [3, 11], [4, 0], [5, 42]]})
    @ddt.unpack
    def test_add_point_and_get_zipped_graph(self, data_stream=None,
                                            zipped_size=None, expected=None):
        merger = utils.GraphZipper(len(data_stream), zipped_size)
        [merger.add_point(value) for value in data_stream]
        self.assertEqual(expected, merger.get_zipped_graph())

    def test_add_point_raises(self):
        merger = utils.GraphZipper(10, 8)
        self.assertRaises(TypeError, merger.add_point)
        [merger.add_point(1) for value in range(10)]
        self.assertRaises(RuntimeError, merger.add_point, 1)



# Copyright 2015: Mirantis Inc.
# All Rights Reserved.
#
#    Licensed under the Apache License, Version 2.0 (the "License"); you may
#    not use this file except in compliance with the License. You may obtain
#    a copy of the License at
#
#         http://www.apache.org/licenses/LICENSE-2.0
#
#    Unless required by applicable law or agreed to in writing, software
#    distributed under the License is distributed on an "AS IS" BASIS, WITHOUT
#    WARRANTIES OR CONDITIONS OF ANY KIND, either express or implied. See the
#    License for the specific language governing permissions and limitations
#    under the License.

NODE_PROCESSES = {
    "vanilla": {
        "1.2.1": {
            "master": ["namenode", "jobtracker", "oozie"],
            "worker": ["datanode", "tasktracker"]
        },
        "2.3.0": {
            "master": ["namenode", "resourcemanager", "historyserver",
                       "oozie"],
            "worker": ["datanode", "nodemanager"]
        },
        "2.4.1": {
            "master": ["namenode", "resourcemanager", "historyserver",
                       "oozie"],
            "worker": ["datanode", "nodemanager"]
        },
        "2.6.0": {
            "master": ["namenode", "resourcemanager", "historyserver",
                       "oozie"],
            "worker": ["datanode", "nodemanager"]
        },
        "2.7.1": {
            "master": ["namenode", "resourcemanager", "historyserver",
                       "oozie"],
            "worker": ["datanode", "nodemanager"]
        }
    },
    "hdp": {
        "1.3.2": {
            "master": ["JOBTRACKER", "NAMENODE", "SECONDARY_NAMENODE",
                       "GANGLIA_SERVER", "NAGIOS_SERVER",
                       "AMBARI_SERVER", "OOZIE_SERVER"],
            "worker": ["TASKTRACKER", "DATANODE", "HDFS_CLIENT",
                       "MAPREDUCE_CLIENT", "OOZIE_CLIENT", "PIG"]
        },
        "2.0.6": {
            "manager": ["AMBARI_SERVER", "GANGLIA_SERVER",
                        "NAGIOS_SERVER"],
            "master": ["NAMENODE", "SECONDARY_NAMENODE",
                       "ZOOKEEPER_SERVER", "ZOOKEEPER_CLIENT",
                       "HISTORYSERVER", "RESOURCEMANAGER",
                       "OOZIE_SERVER"],
            "worker": ["DATANODE", "HDFS_CLIENT", "ZOOKEEPER_CLIENT",
                       "PIG", "MAPREDUCE2_CLIENT", "YARN_CLIENT",
                       "NODEMANAGER", "OOZIE_CLIENT"]
        },
        "2.2": {
            "manager": ["AMBARI_SERVER", "GANGLIA_SERVER",
                        "NAGIOS_SERVER"],
            "master": ["NAMENODE", "SECONDARY_NAMENODE",
                       "ZOOKEEPER_SERVER", "ZOOKEEPER_CLIENT",
                       "HISTORYSERVER", "RESOURCEMANAGER",
                       "OOZIE_SERVER"],
            "worker": ["DATANODE", "HDFS_CLIENT", "ZOOKEEPER_CLIENT",
                       "PIG", "MAPREDUCE2_CLIENT", "YARN_CLIENT",
                       "NODEMANAGER", "OOZIE_CLIENT", "TEZ_CLIENT"]
        }
    },
    "cdh": {
        "5": {
            "manager": ["CLOUDERA_MANAGER"],
            "master": ["HDFS_NAMENODE", "YARN_RESOURCEMANAGER",
                       "OOZIE_SERVER", "YARN_JOBHISTORY",
                       "HDFS_SECONDARYNAMENODE", "HIVE_METASTORE",
                       "HIVE_SERVER2"],
            "worker": ["YARN_NODEMANAGER", "HDFS_DATANODE"]
        },
        "5.4.0": {
            "manager": ["CLOUDERA_MANAGER"],
            "master": ["HDFS_NAMENODE", "YARN_RESOURCEMANAGER",
                       "OOZIE_SERVER", "YARN_JOBHISTORY",
                       "HDFS_SECONDARYNAMENODE", "HIVE_METASTORE",
                       "HIVE_SERVER2"],
            "worker": ["YARN_NODEMANAGER", "HDFS_DATANODE"]
        }
    },
    "spark": {
        "1.3.1": {
            "master": ["namenode", "master"],
            "worker": ["datanode", "slave"]
        }
    },
    "ambari": {
        "2.3": {
            "master-edp": ["Hive Metastore", "HiveServer", "Oozie"],
            "master": ["Ambari", "MapReduce History Server",
                       "Spark History Server", "NameNode", "ResourceManager",
                       "SecondaryNameNode", "YARN Timeline Server",
                       "ZooKeeper"],
            "worker": ["DataNode", "NodeManager"]
        }
    }
}

REPLICATION_CONFIGS = {
    "vanilla": {
        "1.2.1": {
            "target": "HDFS",
            "config_name": "dfs.replication"
        },
        "2.3.0": {
            "target": "HDFS",
            "config_name": "dfs.replication"
        },
        "2.4.1": {
            "target": "HDFS",
            "config_name": "dfs.replication"
        },
        "2.6.0": {
            "target": "HDFS",
            "config_name": "dfs.replication"
        },
        "2.7.1": {
            "target": "HDFS",
            "config_name": "dfs.replication"
        }
    },
    "hdp": {
        "1.3.2": {
            "target": "HDFS",
            "config_name": "dfs.replication"
        },
        "2.0.6": {
            "target": "HDFS",
            "config_name": "dfs.replication"
        },
        "2.2": {
            "target": "HDFS",
            "config_name": "dfs.replication"
        }
    },
    "cdh": {
        "5": {
            "target": "HDFS",
            "config_name": "dfs_replication"
        },
        "5.4.0": {
            "target": "HDFS",
            "config_name": "dfs_replication"
        }
    },
    "spark": {
        "1.3.1": {
            "target": "HDFS",
            "config_name": "dfs_replication"
        },
    },
    "ambari": {
        "2.3": {
            "target": "HDFS",
            "config_name": "dfs_replication"
        },
    }

}

ANTI_AFFINITY_PROCESSES = {
    "vanilla": {
        "1.2.1": ["datanode"],
        "2.3.0": ["datanode"],
        "2.4.1": ["datanode"],
        "2.6.0": ["datanode"],
        "2.7.1": ["datanode"]
    },
    "hdp": {
        "1.3.2": ["DATANODE"],
        "2.0.6": ["DATANODE"],
        "2.2": ["DATANODE"]
    },
    "cdh": {
        "5": ["HDFS_DATANODE"],
        "5.4.0": ["HDFS_DATANODE"]
    },
    "spark": {
        "1.3.1": ["datanode"],
    },
    "ambari": {
        "2.3": ["DataNode"],
    }
}



# Copyright 2014: Mirantis Inc.
# All Rights Reserved.
#
#    Licensed under the Apache License, Version 2.0 (the "License"); you may
#    not use this file except in compliance with the License. You may obtain
#    a copy of the License at
#
#         http://www.apache.org/licenses/LICENSE-2.0
#
#    Unless required by applicable law or agreed to in writing, software
#    distributed under the License is distributed on an "AS IS" BASIS, WITHOUT
#    WARRANTIES OR CONDITIONS OF ANY KIND, either express or implied. See the
#    License for the specific language governing permissions and limitations
#    under the License.

""" Rally command: verify """

import csv
import json
import os

import six
import yaml

from rally import api
from rally.cli import cliutils
from rally.cli import envutils
from rally.common import fileutils
from rally.common.i18n import _
from rally.common import utils
from rally import consts
from rally import exceptions
from rally.verification.tempest import diff
from rally.verification.tempest import json2html


AVAILABLE_SETS = list(consts.TempestTestsSets) + list(consts.TempestTestsAPI)


class VerifyCommands(object):
    """Verify an OpenStack cloud via Tempest.

    Set of commands that allow you to run Tempest tests.
    """

    @cliutils.args("--deployment", dest="deployment", type=str,
                   metavar="<uuid>", required=False,
                   help="UUID or name of a deployment.")
    @cliutils.args("--set", metavar="<set_name>", dest="set_name", type=str,
                   required=False,
                   help="Name of a Tempest test set. "
                        "Available sets are %s" % ", ".join(AVAILABLE_SETS))
    @cliutils.args("--regex", dest="regex", type=str, required=False,
                   help="Regular expression of test")
    @cliutils.args("--tests-file", metavar="<path>", dest="tests_file",
                   type=str, required=False,
                   help="Path to a file with a list of Tempest tests")
    @cliutils.args("--tempest-config", dest="tempest_config", type=str,
                   required=False, metavar="<path>",
                   help="User specified Tempest config file location")
    @cliutils.args("--xfails-file", dest="xfails_file", type=str,
                   required=False, metavar="<path>",
                   help="Path to a file in YAML format with a list of Tempest "
                        "tests that are expected to fail")
    @cliutils.args("--no-use", action="store_false", dest="do_use",
                   help="Don't set new task as default for future operations")
    @cliutils.args("--system-wide", dest="system_wide",
                   help="Don't create a virtual env when installing Tempest; "
                        "use the local env instead of the Tempest virtual env "
                        "when running the tests. Take notice that all Tempest "
                        "requirements have to be already installed in "
                        "the local env!",
                   required=False, action="store_true")
    @cliutils.deprecated_args("--system-wide-install", dest="system_wide",
                              required=False, action="store_true",
                              release="0.1.2", alternative="--system-wide")
    @cliutils.args("--concurrency", metavar="N", dest="concur", type=int,
                   required=False,
                   help="How many processes to use to run Tempest tests. "
                        "The default value (0) auto-detects your CPU count")
    @envutils.with_default_deployment(cli_arg_name="deployment")
    def start(self, deployment=None, set_name="", regex=None,
              tests_file=None, tempest_config=None, xfails_file=None,
              do_use=True, system_wide=False, concur=0):
        """Start verification (run Tempest tests).

        :param deployment: UUID or name of a deployment
        :param set_name: Name of a Tempest test set
        :param regex: Regular expression of test
        :param tests_file: Path to a file with a list of Tempest tests
        :param tempest_config: User specified Tempest config file location
        :param xfails_file: Path to a file in YAML format with a list of
                            Tempest tests that are expected to fail
        :param do_use: Use new task as default for future operations
        :param system_wide: Whether or not to create a virtual env when
                            installing Tempest; whether or not to use
                            the local env instead of the Tempest virtual
                            env when running the tests
        :param concur: How many processes to use to run Tempest tests.
                       The default value (0) auto-detects CPU count
        """
        msg = _("Arguments '%s' and '%s' are not compatible. "
                "You can use only one of the mentioned arguments.")
        if regex and set_name:
            print(msg % ("regex", "set"))
            return 1
        if tests_file and set_name:
            print(msg % ("tests_file", "set"))
            return 1
        if tests_file and regex:
            print(msg % ("tests_file", "regex"))
            return 1

        if not (regex or set_name or tests_file):
            set_name = "full"

        if set_name and set_name not in AVAILABLE_SETS:
            print(_("Tempest test set '%s' not found "
                    "in available test sets. Available sets are %s.")
                  % (set_name, ", ".join(AVAILABLE_SETS)))
            return 1

        if tests_file and not os.path.exists(tests_file):
            print(_("File '%s' not found.") % tests_file)
            return 1

        expected_failures = None
        if xfails_file:
            if os.path.exists(xfails_file):
                with open(os.path.abspath(xfails_file), "rb") as f:
                    expected_failures = yaml.load(f)
            else:
                print(_("File '%s' not found.") % xfails_file)
                return 1

        verification = api.Verification.verify(
            deployment, set_name=set_name, regex=regex, tests_file=tests_file,
            tempest_config=tempest_config, expected_failures=expected_failures,
            system_wide=system_wide, concur=concur)
        if do_use:
            self.use(verification["uuid"])

    @cliutils.args("--deployment", dest="deployment", type=str,
                   metavar="<uuid>", required=False,
                   help="UUID or name of a deployment.")
    @cliutils.args("--set", metavar="<set_name>", dest="set_name", type=str,
                   required=False,
                   help="Name of a Tempest test set. "
                        "Available sets are %s" % ", ".join(AVAILABLE_SETS))
    @cliutils.args("--file", dest="log_file", type=str,
                   required=True, metavar="<path>",
                   help="User specified Tempest log file location. "
                        "Note, Tempest log file needs to be in subunit format")
    @cliutils.args("--no-use", action="store_false", dest="do_use",
                   required=False,
                   help="Don't set new task as default for future operations")
    @cliutils.alias("import")
    @envutils.with_default_deployment(cli_arg_name="deployment")
    def import_results(self, deployment=None, set_name="", log_file=None,
                       do_use=True):
        """Import Tempest tests results into the Rally database.

        :param deployment: UUID or name of a deployment
        :param set_name: Name of a Tempest test set
        :param do_use: Use new task as default for future operations
        :param log_file: User specified Tempest log file in subunit format
        """
        deployment, verification = api.Verification.import_results(deployment,
                                                                   set_name,
                                                                   log_file)
        if do_use:
            self.use(verification["uuid"])

    def list(self):
        """Display verifications table."""

        fields = ["UUID", "Deployment UUID", "Set name", "Tests", "Failures",
                  "Created at", "Duration", "Status"]
        verifications = api.Verification.list()

        for el in verifications:
            el["duration"] = el["updated_at"] - el["created_at"]

        if verifications:
            cliutils.print_list(verifications, fields,
                                sortby_index=fields.index("Created at"))
        else:
            print(_("No verification was started yet. "
                    "To start verification use:\nrally verify start"))

    @cliutils.args("--uuid", type=str, dest="verification",
                   help="UUID of a verification")
    @cliutils.args("--html", action="store_true", dest="output_html",
                   help="Display results in HTML format")
    @cliutils.args("--json", action="store_true", dest="output_json",
                   help="Display results in JSON format")
    @cliutils.args("--output-file", type=str, required=False,
                   dest="output_file", metavar="<path>",
                   help="Path to a file to save results")
    @envutils.with_default_verification_id
    @cliutils.suppress_warnings
    def results(self, verification=None, output_file=None,
                output_html=None, output_json=None):
        """Display results of a verification.

        :param verification: UUID of a verification
        :param output_file: Path to a file to save results
        :param output_html: Display results in HTML format
        :param output_json: Display results in JSON format (Default)
        """
        try:
            results = api.Verification.get(verification).get_results()
        except exceptions.NotFoundException as e:
            print(six.text_type(e))
            return 1

        result = ""
        if output_json + output_html > 1:
            print(_("Please specify only one "
                    "output format: --json or --html."))
        elif output_html:
            result = json2html.generate_report(results)
        else:
            result = json.dumps(results, sort_keys=True, indent=4)

        if output_file:
            output_file = os.path.expanduser(output_file)
            with open(output_file, "wb") as f:
                f.write(result)
        else:
            print(result)

    @cliutils.args("--uuid", dest="verification", type=str,
                   required=False,
                   help="UUID of a verification")
    @cliutils.args("--sort-by", metavar="<query>", dest="sort_by", type=str,
                   required=False,
                   help="Sort results by 'name' or 'duration'")
    @cliutils.args("--detailed", dest="detailed", action="store_true",
                   required=False,
                   help="Display detailed errors of failed tests")
    @envutils.with_default_verification_id
    def show(self, verification=None, sort_by="name", detailed=False):
        """Display results table of a verification.

        :param verification: UUID of a verification
        :param sort_by: Sort results by 'name' or 'duration'
        :param detailed: Display detailed errors of failed tests
        """
        try:
            sortby_index = ("name", "duration").index(sort_by)
        except ValueError:
            print(_("Verification results can't be sorted by '%s'.") % sort_by)
            return 1

        try:
            verification = api.Verification.get(verification)
            tests = verification.get_results()
        except exceptions.NotFoundException as e:
            print(six.text_type(e))
            return 1

        print(_("Total results of verification:\n"))
        total_fields = ["UUID", "Deployment UUID", "Set name", "Tests",
                        "Failures", "Created at", "Status"]
        cliutils.print_list([verification], fields=total_fields)

        print(_("\nTests:\n"))
        fields = ["name", "time", "status"]

        results = tests["test_cases"]
        values = [utils.Struct(**results[test_name]) for test_name in results]
        cliutils.print_list(values, fields, sortby_index=sortby_index)

        if detailed:
            for test in six.itervalues(tests["test_cases"]):
                if test["status"] == "fail":
                    header = cliutils.make_header(
                        "FAIL: %(name)s\n"
                        "Time: %(time)s" % {"name": test["name"],
                                            "time": test["time"]})
                    formatted_test = "%(header)s%(log)s\n" % {
                        "header": header,
                        "log": test["traceback"]}
                    print(formatted_test)

    @cliutils.args("--uuid", dest="verification", type=str,
                   required=False, help="UUID of a verification")
    @cliutils.args("--sort-by", dest="sort_by", type=str, required=False,
                   help="Sort results by 'name' or 'duration'")
    @envutils.with_default_verification_id
    def detailed(self, verification=None, sort_by="name"):
        """Display results table of a verification with detailed errors.

        :param verification: UUID of a verification
        :param sort_by: Sort results by 'name' or 'duration'
        """
        self.show(verification, sort_by, True)

    @cliutils.args("--uuid-1", type=str, required=True, dest="verification1",
                   help="UUID of the first verification")
    @cliutils.args("--uuid-2", type=str, required=True, dest="verification2",
                   help="UUID of the second verification")
    @cliutils.args("--csv", action="store_true", dest="output_csv",
                   help="Display results in CSV format")
    @cliutils.args("--html", action="store_true", dest="output_html",
                   help="Display results in HTML format")
    @cliutils.args("--json", action="store_true", dest="output_json",
                   help="Display results in JSON format")
    @cliutils.args("--output-file", type=str, required=False,
                   dest="output_file", help="Path to a file to save results")
    @cliutils.args("--threshold", type=int, required=False,
                   dest="threshold", default=0,
                   help="If specified, timing differences must exceed this "
                   "percentage threshold to be included in output")
    def compare(self, verification1=None, verification2=None,
                output_file=None, output_csv=None, output_html=None,
                output_json=None, threshold=0):
        """Compare two verification results.

        :param verification1: UUID of the first verification
        :param verification2: UUID of the second verification
        :param output_file: Path to a file to save results
        :param output_csv: Display results in CSV format
        :param output_html: Display results in HTML format
        :param output_json: Display results in JSON format (Default)
        :param threshold: Timing difference threshold percentage
        """
        try:
            res_1 = api.Verification.get(
                verification1).get_results()["test_cases"]
            res_2 = api.Verification.get(
                verification2).get_results()["test_cases"]
            _diff = diff.Diff(res_1, res_2, threshold)
        except exceptions.NotFoundException as e:
            print(six.text_type(e))
            return 1

        result = ""
        if output_json + output_html + output_csv > 1:
            print(_("Please specify only one output "
                    "format: --json, --html or --csv."))
            return 1
        elif output_html:
            result = _diff.to_html()
        elif output_csv:
            result = _diff.to_csv()
        else:
            result = _diff.to_json()

        if output_file:
            with open(output_file, "wb") as f:
                if output_csv:
                    writer = csv.writer(f, dialect="excel")
                    writer.writerows(result)
                else:
                    f.write(result)
        else:
            print(result)

    @cliutils.args("--uuid", type=str, dest="verification",
                   required=False, help="UUID of a verification")
    @cliutils.deprecated_args("--verification", dest="verification",
                              required=False, type=str,
                              release="0.2.0",
                              alternative="--uuid")
    def use(self, verification):
        """Set active verification.

        :param verification: UUID of a verification
        """
        print(_("Verification UUID: %s") % verification)
        api.Verification.get(verification)
        fileutils.update_globals_file("RALLY_VERIFICATION", verification)

    @cliutils.args("--deployment", dest="deployment", type=str,
                   metavar="<uuid>", required=False,
                   help="UUID or name of a deployment.")
    @cliutils.args("--tempest-config", dest="tempest_config", type=str,
                   required=False, metavar="<path>",
                   help="User specified Tempest config file location")
    @cliutils.args("--override", dest="override",
                   help="Override existing Tempest config file",
                   required=False, action="store_true")
    @envutils.with_default_deployment(cli_arg_name="deployment")
    def genconfig(self, deployment=None, tempest_config=None, override=False):
        """Generate configuration file of Tempest.

        :param deployment: UUID or name of a deployment
        :param tempest_config: User specified Tempest config file location
        :param override: Whether or not to override existing Tempest
                         config file
        """
        api.Verification.configure_tempest(deployment, tempest_config,
                                           override)

    @cliutils.args("--deployment", dest="deployment", type=str,
                   metavar="<uuid>", required=False,
                   help="UUID or name of a deployment.")
    @cliutils.args("--source", type=str, dest="source", required=False,
                   help="Path/URL to repo to clone Tempest from")
    @cliutils.args("--no-tempest-venv", dest="no_tempest_venv",
                   help="Don't create a virtual env for Tempest. Take notice "
                        "that all Tempest requirements have to be already "
                        "installed in the local env!",
                   required=False, action="store_true")
    @envutils.with_default_deployment(cli_arg_name="deployment")
    def install(self, deployment=None, source=None, no_tempest_venv=False):
        """Install Tempest.

        :param deployment: UUID or name of a deployment
        :param source: Path/URL to repo to clone Tempest from
        :param no_tempest_venv: Whether or not to create a Tempest virtual env
        """
        api.Verification.install_tempest(deployment, source, no_tempest_venv)

    @cliutils.args("--deployment", dest="deployment", type=str,
                   metavar="<uuid>", required=False,
                   help="UUID or name of a deployment.")
    @envutils.with_default_deployment(cli_arg_name="deployment")
    def uninstall(self, deployment=None):
        """Remove deployment's local Tempest installation.

        :param deployment: UUID or name of a deployment
        """
        api.Verification.uninstall_tempest(deployment)

    @cliutils.args("--deployment", dest="deployment", type=str,
                   metavar="<uuid>", required=False,
                   help="UUID or name of a deployment.")
    @cliutils.args("--tempest-config", dest="tempest_config", type=str,
                   required=False, metavar="<path>",
                   help="User specified Tempest config file location")
    @cliutils.args("--source", type=str, dest="source", required=False,
                   help="Path/URL to repo to clone Tempest from")
    @cliutils.args("--no-tempest-venv", dest="no_tempest_venv",
                   help="Don't create a virtual env for Tempest. Take notice "
                        "that all Tempest requirements have to be already "
                        "installed in the local env!",
                   required=False, action="store_true")
    @envutils.with_default_deployment(cli_arg_name="deployment")
    def reinstall(self, deployment=None,
                  tempest_config=None, source=None, no_tempest_venv=False):
        """Uninstall Tempest and install again.

        :param deployment: UUID or name of a deployment
        :param tempest_config: User specified Tempest config file location
        :param source: Path/URL to repo to clone Tempest from
        :param no_tempest_venv: Whether or not to create a Tempest virtual env
        """
        api.Verification.reinstall_tempest(deployment, tempest_config,
                                           source, no_tempest_venv)

    @cliutils.args("--deployment", dest="deployment", type=str,
                   metavar="<uuid>", required=False,
                   help="UUID or name of a deployment.")
    @envutils.with_default_deployment(cli_arg_name="deployment")
    def showconfig(self, deployment=None):
        """Show configuration file of Tempest.

        :param deployment: UUID or name of a deployment
        """
        conf_info = api.Verification.show_config_info(deployment)
        print(_("Tempest config file: %s") % conf_info["conf_path"])
        print("\n" + conf_info["conf_data"])



# All Rights Reserved.
#
# Licensed under the Apache License, Version 2.0 (the "License"); you may
# not use this file except in compliance with the License. You may obtain
# a copy of the License at
#
# http://www.apache.org/licenses/LICENSE-2.0
#
# Unless required by applicable law or agreed to in writing, software
# distributed under the License is distributed on an "AS IS" BASIS, WITHOUT
# WARRANTIES OR CONDITIONS OF ANY KIND, either express or implied. See the
# License for the specific language governing permissions and limitations
# under the License.


import copy

import mock

from rally.plugins.openstack.context.designate import zones
from tests.unit import test

CTX = "rally.plugins.openstack.context"
SCN = "rally.plugins.openstack.scenarios"


class ZoneGeneratorTestCase(test.ScenarioTestCase):

    def _gen_tenants(self, count):
        tenants = {}
        for id_ in range(count):
            tenants[str(id_)] = {"name": str(id_)}
        return tenants

    def test_init(self):
        self.context.update({
            "config": {
                "zones": {
                    "zones_per_tenant": 5,
                }
            }
        })

        inst = zones.ZoneGenerator(self.context)
        self.assertEqual(inst.config, self.context["config"]["zones"])

    @mock.patch("%s.designate.utils.DesignateScenario._create_zone" % SCN,
                return_value={"id": "uuid"})
    def test_setup(self, mock_designate_scenario__create_zone):
        tenants_count = 2
        users_per_tenant = 5
        zones_per_tenant = 5

        tenants = self._gen_tenants(tenants_count)
        users = []
        for id_ in tenants.keys():
            for i in range(users_per_tenant):
                users.append({"id": i, "tenant_id": id_,
                              "credential": mock.MagicMock()})

        self.context.update({
            "config": {
                "users": {
                    "tenants": 2,
                    "users_per_tenant": 5,
                    "concurrent": 10,
                },
                "zones": {
                    "zones_per_tenant": zones_per_tenant,
                }
            },
            "admin": {
                "credential": mock.MagicMock()
            },
            "users": users,
            "tenants": tenants
        })

        new_context = copy.deepcopy(self.context)
        for id_ in tenants.keys():
            new_context["tenants"][id_].setdefault("zones", [])
            for i in range(zones_per_tenant):
                new_context["tenants"][id_]["zones"].append({"id": "uuid"})

        zones_ctx = zones.ZoneGenerator(self.context)
        zones_ctx.setup()
        self.assertEqual(new_context, self.context)

    @mock.patch("%s.designate.zones.resource_manager.cleanup" % CTX)
    def test_cleanup(self, mock_cleanup):

        tenants_count = 2
        users_per_tenant = 5
        zones_per_tenant = 5

        tenants = self._gen_tenants(tenants_count)
        users = []
        for id_ in tenants.keys():
            for i in range(users_per_tenant):
                users.append({"id": i, "tenant_id": id_,
                              "endpoint": "endpoint"})
            tenants[id_].setdefault("zones", [])
            for j in range(zones_per_tenant):
                tenants[id_]["zones"].append({"id": "uuid"})

        self.context.update({
            "config": {
                "users": {
                    "tenants": 2,
                    "users_per_tenant": 5,
                    "concurrent": 10,
                },
                "zones": {
                    "zones_per_tenant": 5,
                }
            },
            "admin": {
                "endpoint": mock.MagicMock()
            },
            "users": users,
            "tenants": tenants
        })

        zones_ctx = zones.ZoneGenerator(self.context)
        zones_ctx.cleanup()

        mock_cleanup.assert_called_once_with(names=["designate.zones"],
                                             users=self.context["users"])



# All Rights Reserved.
#
#    Licensed under the Apache License, Version 2.0 (the "License"); you may
#    not use this file except in compliance with the License. You may obtain
#    a copy of the License at
#
#         http://www.apache.org/licenses/LICENSE-2.0
#
#    Unless required by applicable law or agreed to in writing, software
#    distributed under the License is distributed on an "AS IS" BASIS, WITHOUT
#    WARRANTIES OR CONDITIONS OF ANY KIND, either express or implied. See the
#    License for the specific language governing permissions and limitations
#    under the License.

import random

from rally.plugins.openstack import scenario
from rally.plugins.openstack.scenarios.fuel import utils
from rally.task import validation


class FuelNodes(utils.FuelScenario):
    """Benchmark scenarios for Fuel nodes."""

    @validation.required_clients("fuel", admin=True)
    @validation.required_openstack(admin=True)
    @validation.required_contexts("fuel_environments")
    @scenario.configure()
    def add_and_remove_node(self, node_roles=None):
        """Add node to environment and remove

        :param node_roles: list. Roles, which node should be assigned to
            env with
        """

        env_id = random.choice(self.context["fuel"]["environments"])

        node_id = self._get_free_node_id()
        self._add_node(env_id, [node_id], node_roles)
        self._remove_node(env_id, node_id)



# Copyright 2014: Mirantis Inc.
# All Rights Reserved.
#
#    Licensed under the Apache License, Version 2.0 (the "License"); you may
#    not use this file except in compliance with the License. You may obtain
#    a copy of the License at
#
#         http://www.apache.org/licenses/LICENSE-2.0
#
#    Unless required by applicable law or agreed to in writing, software
#    distributed under the License is distributed on an "AS IS" BASIS, WITHOUT
#    WARRANTIES OR CONDITIONS OF ANY KIND, either express or implied. See the
#    License for the specific language governing permissions and limitations
#    under the License.

import os

import ddt
import mock
from oslo_config import cfg
import requests
from six.moves.urllib import parse

from rally import exceptions
from rally.verification.tempest import config
from tests.unit import fakes
from tests.unit import test

CONF = cfg.CONF


CREDS = {
    "admin": {
        "username": "admin",
        "tenant_name": "admin",
        "password": "admin-12345",
        "auth_url": "http://test/v2.0/",
        "permission": "admin",
        "region_name": "test",
        "admin_domain_name": "Default",
        "https_insecure": False,
        "https_cacert": "/path/to/cacert/file"
    }
}


@ddt.ddt
class TempestConfigTestCase(test.TestCase):

    def setUp(self):
        super(TempestConfigTestCase, self).setUp()

        mock.patch("rally.common.objects.deploy.db.deployment_get",
                   return_value=CREDS).start()
        mock.patch("rally.osclients.Clients").start()
        self.mock_isfile = mock.patch("os.path.isfile",
                                      return_value=True).start()

        self.tempest_conf = config.TempestConfig("fake_deployment")

    @mock.patch("os.rename")
    @mock.patch("six.moves.builtins.open", side_effect=mock.mock_open())
    @mock.patch("requests.get", return_value=mock.MagicMock(status_code=200))
    def test__download_cirros_image_success(self, mock_get,
                                            mock_open, mock_rename):
        self.mock_isfile.return_value = False
        self.tempest_conf._download_cirros_image()
        mock_get.assert_called_once_with(
            CONF.image.cirros_img_url, stream=True)

    @mock.patch("requests.get")
    @ddt.data(404, 500)
    def test__download_cirros_image_failure(self, status_code, mock_get):
        self.mock_isfile.return_value = False
        mock_get.return_value = mock.MagicMock(status_code=status_code)
        self.assertRaises(exceptions.TempestConfigCreationFailure,
                          self.tempest_conf._download_cirros_image)

    @mock.patch("requests.get", side_effect=requests.ConnectionError())
    def test__download_cirros_image_connection_error(self, mock_requests_get):
        self.mock_isfile.return_value = False
        self.assertRaises(exceptions.TempestConfigCreationFailure,
                          self.tempest_conf._download_cirros_image)

    def test__get_service_url(self):
        self.tempest_conf.keystone.auth_ref = {
            "serviceCatalog": [
                {
                    "name": "test_service",
                    "type": "test_service_type",
                    "endpoints": [{"publicURL": "test_url"}]
                }
            ]
        }
        self.tempest_conf.clients.services.return_value = {
            "test_service_type": "test_service"}
        self.assertEqual(
            self.tempest_conf._get_service_url("test_service"), "test_url")

    @mock.patch("rally.verification.tempest.config."
                "TempestConfig._get_service_url", return_value="test_url")
    def test__configure_boto(self, mock__get_service_url):
        self.tempest_conf._configure_boto()

        expected = (("ec2_url", "test_url"),
                    ("s3_url", "test_url"),
                    ("http_socket_timeout", "30"),
                    ("s3_materials_path", os.path.join(
                        self.tempest_conf.data_dir, "s3materials")))
        result = self.tempest_conf.conf.items("boto")
        for item in expected:
            self.assertIn(item, result)

    def test__configure_default(self):
        self.tempest_conf._configure_default()

        expected = (("debug", "True"),
                    ("log_file", "tempest.log"),
                    ("use_stderr", "False"))
        results = self.tempest_conf.conf.items("DEFAULT")
        self.assertEqual(sorted(expected), sorted(results))

    def test__configure_dashboard(self):
        self.tempest_conf._configure_dashboard()
        url = "http://%s/" % parse.urlparse(
            CREDS["admin"]["auth_url"]).hostname
        self.assertEqual(
            self.tempest_conf.conf.get("dashboard", "dashboard_url"), url)

    @ddt.data("data_processing", "data-processing")
    def test__configure_data_processing(self, service_type):
        self.tempest_conf.available_services = ["sahara"]

        self.tempest_conf.clients.services.return_value = {
            service_type: "sahara"}
        self.tempest_conf._configure_data_processing()
        self.assertEqual(
            self.tempest_conf.conf.get(
                "data-processing", "catalog_type"), service_type)

    def test__configure_identity(self):
        self.tempest_conf._configure_identity()

        expected = (
            ("username", CREDS["admin"]["username"]),
            ("password", CREDS["admin"]["password"]),
            ("tenant_name", CREDS["admin"]["tenant_name"]),
            ("admin_username", CREDS["admin"]["username"]),
            ("admin_password", CREDS["admin"]["password"]),
            ("admin_tenant_name", CREDS["admin"]["username"]),
            ("admin_domain_name", CREDS["admin"]["admin_domain_name"]),
            ("region", CREDS["admin"]["region_name"]),
            ("uri", CREDS["admin"]["auth_url"]),
            ("uri_v3", CREDS["admin"]["auth_url"].replace("/v2.0/", "/v3")),
            ("disable_ssl_certificate_validation",
             str(CREDS["admin"]["https_insecure"])),
            ("ca_certificates_file", CREDS["admin"]["https_cacert"]))
        result = self.tempest_conf.conf.items("identity")
        for item in expected:
            self.assertIn(item, result)

    def test__configure_network_if_neutron(self):
        self.tempest_conf.available_services = ["neutron"]
        client = self.tempest_conf.clients.neutron()
        client.list_networks.return_value = {
            "networks": [
                {
                    "status": "ACTIVE",
                    "id": "test_id",
                    "router:external": True
                }
            ]
        }

        self.tempest_conf._configure_network()
        self.assertEqual(
            self.tempest_conf.conf.get("network",
                                       "public_network_id"), "test_id")

    def test__configure_network_if_nova(self):
        self.tempest_conf.available_services = ["nova"]
        client = self.tempest_conf.clients.nova()
        client.networks.list.return_value = [
            mock.MagicMock(human_id="fake-network")]

        self.tempest_conf._configure_network()

        expected = (("network_for_ssh", "fake-network"),
                    ("fixed_network_name", "fake-network"))
        result = self.tempest_conf.conf.items("compute")
        for item in expected:
            self.assertIn(item, result)

    def test__configure_network_feature_enabled(self):
        self.tempest_conf.available_services = ["neutron"]
        client = self.tempest_conf.clients.neutron()
        client.list_ext.return_value = {
            "extensions": [
                {"alias": "dvr"},
                {"alias": "extra_dhcp_opt"},
                {"alias": "extraroute"}
            ]
        }

        self.tempest_conf._configure_network_feature_enabled()
        self.assertEqual(self.tempest_conf.conf.get(
            "network-feature-enabled", "api_extensions"),
            "dvr,extra_dhcp_opt,extraroute")

    @mock.patch("os.makedirs")
    @mock.patch("os.path.exists", return_value=False)
    def test__configure_oslo_concurrency(self, mock_exists, mock_makedirs):
        self.tempest_conf._configure_oslo_concurrency()

        lock_path = os.path.join(
            self.tempest_conf.data_dir, "lock_files_fake_deployment")
        mock_makedirs.assert_called_with(lock_path)
        self.assertEqual(
            self.tempest_conf.conf.get(
                "oslo_concurrency", "lock_path"), lock_path)

    def test__configure_object_storage(self):
        self.tempest_conf._configure_object_storage()

        expected = (
            ("operator_role", CONF.role.swift_operator_role),
            ("reseller_admin_role", CONF.role.swift_reseller_admin_role))
        result = self.tempest_conf.conf.items("object-storage")
        for item in expected:
            self.assertIn(item, result)

    def test__configure_orchestration(self):
        self.tempest_conf._configure_orchestration()

        expected = (
            ("stack_owner_role", CONF.role.heat_stack_owner_role),
            ("stack_user_role", CONF.role.heat_stack_user_role))
        result = self.tempest_conf.conf.items("orchestration")
        for item in expected:
            self.assertIn(item, result)

    def test__configure_scenario(self):
        self.tempest_conf._configure_scenario()

        image_name = parse.urlparse(
            config.CONF.image.cirros_img_url).path.split("/")[-1]
        expected = (("img_dir", self.tempest_conf.data_dir),
                    ("img_file", image_name))
        result = self.tempest_conf.conf.items("scenario")
        for item in expected:
            self.assertIn(item, result)

    @mock.patch("requests.get", return_value=mock.MagicMock(status_code=200))
    def test__configure_service_available(self, mock_get):
        available_services = ("nova", "cinder", "glance", "sahara")
        self.tempest_conf.available_services = available_services
        self.tempest_conf._configure_service_available()

        expected_horizon_url = "http://test"
        expected_timeout = CONF.openstack_client_http_timeout
        mock_get.assert_called_once_with(expected_horizon_url,
                                         timeout=expected_timeout)
        expected = (
            ("neutron", "False"), ("heat", "False"), ("nova", "True"),
            ("swift", "False"), ("cinder", "True"), ("sahara", "True"),
            ("glance", "True"), ("horizon", "True"), ("ceilometer", "False"))
        result = self.tempest_conf.conf.items("service_available")
        for item in expected:
            self.assertIn(item, result)

    @mock.patch("requests.get", return_value=mock.MagicMock(status_code=404))
    def test__configure_service_available_horizon_not_available(
            self, mock_get):
        self.tempest_conf._configure_service_available()
        self.assertEqual(
            self.tempest_conf.conf.get(
                "service_available", "horizon"), "False")

    @mock.patch("requests.get", side_effect=requests.Timeout())
    def test__configure_service_available_horizon_request_timeout(
            self, mock_get):
        self.tempest_conf._configure_service_available()
        self.assertEqual(
            self.tempest_conf.conf.get(
                "service_available", "horizon"), "False")

    @ddt.data({}, {"service": "neutron", "connect_method": "floating"})
    @ddt.unpack
    def test__configure_validation(self, service="nova",
                                   connect_method="fixed"):
        self.tempest_conf.available_services = [service]
        self.tempest_conf._configure_validation()

        expected = (("run_validation", "True"),
                    ("connect_method", connect_method))
        result = self.tempest_conf.conf.items("validation")
        for item in expected:
            self.assertIn(item, result)

    @mock.patch("rally.verification.tempest.config._write_config")
    @mock.patch("inspect.getmembers")
    def test_generate(self, mock_inspect_getmembers, mock__write_config):
        configure_something_method = mock.MagicMock()
        mock_inspect_getmembers.return_value = [("_configure_something",
                                                 configure_something_method)]

        self.tempest_conf.generate("/path/to/fake/conf")
        self.assertEqual(configure_something_method.call_count, 1)
        self.assertEqual(mock__write_config.call_count, 1)

    @mock.patch("six.moves.builtins.open", side_effect=mock.mock_open())
    def test__write_config(self, mock_open):
        conf_path = "/path/to/fake/conf"
        conf_data = mock.Mock()

        config._write_config(conf_path, conf_data)
        mock_open.assert_called_once_with(conf_path, "w+")
        conf_data.write.assert_called_once_with(mock_open.side_effect())


class TempestResourcesContextTestCase(test.TestCase):

    def setUp(self):
        super(TempestResourcesContextTestCase, self).setUp()

        mock.patch("rally.common.objects.deploy.db.deployment_get",
                   return_value=CREDS).start()
        mock.patch("rally.osclients.Clients").start()

        fake_verification = {"uuid": "uuid"}
        self.context = config.TempestResourcesContext("fake_deployment",
                                                      fake_verification,
                                                      "/fake/path/to/config")
        self.context.conf.add_section("compute")
        self.context.conf.add_section("orchestration")

    @mock.patch("rally.plugins.openstack.wrappers."
                "network.NeutronWrapper.create_network")
    @mock.patch("six.moves.builtins.open", side_effect=mock.mock_open())
    def test_options_configured_manually(
            self, mock_open, mock_neutron_wrapper_create_network):
        self.context.available_services = ["glance", "heat", "nova", "neutron"]

        self.context.conf.set("compute", "image_ref", "id1")
        self.context.conf.set("compute", "image_ref_alt", "id2")
        self.context.conf.set("compute", "flavor_ref", "id3")
        self.context.conf.set("compute", "flavor_ref_alt", "id4")
        self.context.conf.set("compute", "fixed_network_name", "name1")
        self.context.conf.set("orchestration", "instance_type", "id5")

        self.context.__enter__()

        glanceclient = self.context.clients.glance()
        novaclient = self.context.clients.nova()

        self.assertEqual(glanceclient.images.create.call_count, 0)
        self.assertEqual(novaclient.flavors.create.call_count, 0)
        self.assertEqual(mock_neutron_wrapper_create_network.call_count, 0)

    def test__create_tempest_roles(self):
        role1 = CONF.role.swift_operator_role
        role2 = CONF.role.swift_reseller_admin_role
        role3 = CONF.role.heat_stack_owner_role
        role4 = CONF.role.heat_stack_user_role

        client = self.context.clients.verified_keystone()
        client.roles.list.return_value = [fakes.FakeRole(name=role1),
                                          fakes.FakeRole(name=role2)]
        client.roles.create.side_effect = [fakes.FakeFlavor(name=role3),
                                           fakes.FakeFlavor(name=role4)]

        self.context._create_tempest_roles()
        self.assertEqual(client.roles.create.call_count, 2)

        created_roles = [role.name for role in self.context._created_roles]
        self.assertIn(role3, created_roles)
        self.assertIn(role4, created_roles)

    # We can choose any option to test the '_configure_option' method. So let's
    # configure the 'flavor_ref' option.
    def test__configure_option(self):
        create_method = mock.MagicMock()
        create_method.side_effect = [fakes.FakeFlavor(id="id1")]

        self.context.conf.set("compute", "flavor_ref", "")
        self.context._configure_option("compute",
                                       "flavor_ref", create_method, 64)
        self.assertEqual(create_method.call_count, 1)

        result = self.context.conf.get("compute", "flavor_ref")
        self.assertEqual("id1", result)

    @mock.patch("six.moves.builtins.open")
    def test__create_image(self, mock_open):
        client = self.context.clients.glance()
        client.images.create.side_effect = [fakes.FakeImage(id="id1")]

        image = self.context._create_image()
        self.assertEqual("id1", image.id)
        self.assertEqual("id1", self.context._created_images[0].id)

    def test__create_flavor(self):
        client = self.context.clients.nova()
        client.flavors.create.side_effect = [fakes.FakeFlavor(id="id1")]

        flavor = self.context._create_flavor(64)
        self.assertEqual("id1", flavor.id)
        self.assertEqual("id1", self.context._created_flavors[0].id)

    def test__create_network_resources(self):
        client = self.context.clients.neutron()
        fake_network = {
            "id": "nid1",
            "name": "network",
            "status": "status"}

        client.create_network.side_effect = [{"network": fake_network}]
        client.create_router.side_effect = [{"router": {"id": "rid1"}}]
        client.create_subnet.side_effect = [{"subnet": {"id": "subid1"}}]

        network = self.context._create_network_resources()
        self.assertEqual("nid1", network["id"])
        self.assertEqual("nid1", self.context._created_networks[0]["id"])
        self.assertEqual("rid1",
                         self.context._created_networks[0]["router_id"])
        self.assertEqual("subid1",
                         self.context._created_networks[0]["subnets"][0])

    def test__cleanup_tempest_roles(self):
        self.context._created_roles = [fakes.FakeRole(), fakes.FakeRole()]

        self.context._cleanup_tempest_roles()
        client = self.context.clients.keystone()
        self.assertEqual(client.roles.delete.call_count, 2)

    def test__cleanup_images(self):
        self.context._created_images = [fakes.FakeImage(id="id1"),
                                        fakes.FakeImage(id="id2")]

        self.context.conf.set("compute", "image_ref", "id1")
        self.context.conf.set("compute", "image_ref_alt", "id2")

        self.context._cleanup_images()
        client = self.context.clients.glance()
        self.assertEqual(client.images.delete.call_count, 2)

        self.assertEqual("", self.context.conf.get("compute", "image_ref"))
        self.assertEqual("", self.context.conf.get("compute", "image_ref_alt"))

    def test__cleanup_flavors(self):
        self.context._created_flavors = [fakes.FakeFlavor(id="id1"),
                                         fakes.FakeFlavor(id="id2"),
                                         fakes.FakeFlavor(id="id3")]

        self.context.conf.set("compute", "flavor_ref", "id1")
        self.context.conf.set("compute", "flavor_ref_alt", "id2")
        self.context.conf.set("orchestration", "instance_type", "id3")

        self.context._cleanup_flavors()
        client = self.context.clients.nova()
        self.assertEqual(client.flavors.delete.call_count, 3)

        self.assertEqual("", self.context.conf.get("compute", "flavor_ref"))
        self.assertEqual("", self.context.conf.get("compute",
                                                   "flavor_ref_alt"))
        self.assertEqual("", self.context.conf.get("orchestration",
                                                   "instance_type"))

    @mock.patch("rally.plugins.openstack.wrappers."
                "network.NeutronWrapper.delete_network")
    def test__cleanup_network_resources(
            self, mock_neutron_wrapper_delete_network):
        self.context._created_networks = [{"name": "net-12345"}]
        self.context.conf.set("compute", "fixed_network_name", "net-12345")

        self.context._cleanup_network_resources()
        self.assertEqual(mock_neutron_wrapper_delete_network.call_count, 1)
        self.assertEqual("", self.context.conf.get("compute",
                                                   "fixed_network_name"))



# Copyright 2014: Intel Inc.
# All Rights Reserved.
#
#    Licensed under the Apache License, Version 2.0 (the "License"); you may
#    not use this file except in compliance with the License. You may obtain
#    a copy of the License at
#
#         http://www.apache.org/licenses/LICENSE-2.0
#
#    Unless required by applicable law or agreed to in writing, software
#    distributed under the License is distributed on an "AS IS" BASIS, WITHOUT
#    WARRANTIES OR CONDITIONS OF ANY KIND, either express or implied. See the
#    License for the specific language governing permissions and limitations
#    under the License.

import ddt
import mock

from rally.plugins.openstack.scenarios.neutron import network
from tests.unit import test

NEUTRON_NETWORKS = ("rally.plugins.openstack.scenarios.neutron.network"
                    ".NeutronNetworks")


@ddt.ddt
class NeutronNetworksTestCase(test.ScenarioTestCase):

    @mock.patch(NEUTRON_NETWORKS + "._list_networks")
    @mock.patch(NEUTRON_NETWORKS + "._create_network")
    def test_create_and_list_networks(self, mock__create_network,
                                      mock__list_networks):
        neutron_scenario = network.NeutronNetworks(self.context)

        # Default options
        network_create_args = {}
        neutron_scenario.create_and_list_networks(
            network_create_args=network_create_args)
        mock__create_network.assert_called_once_with(network_create_args)
        mock__list_networks.assert_called_once_with()

        mock__create_network.reset_mock()
        mock__list_networks.reset_mock()

        # Explicit network name is specified
        network_create_args = {"name": "given-name"}
        neutron_scenario.create_and_list_networks(
            network_create_args=network_create_args)
        mock__create_network.assert_called_once_with(network_create_args)
        mock__list_networks.assert_called_once_with()

    @mock.patch(NEUTRON_NETWORKS + "._update_network")
    @mock.patch(NEUTRON_NETWORKS + "._create_network", return_value={
        "network": {
            "id": "network-id",
            "name": "network-name",
            "admin_state_up": False
        }
    })
    def test_create_and_update_networks(self,
                                        mock__create_network,
                                        mock__update_network):
        scenario = network.NeutronNetworks(self.context)

        network_update_args = {"name": "_updated", "admin_state_up": True}

        # Default options
        scenario.create_and_update_networks(
            network_update_args=network_update_args)

        mock__create_network.assert_called_once_with({})

        mock__update_network.assert_has_calls(
            [mock.call(
                mock__create_network.return_value, network_update_args
            )])

        mock__create_network.reset_mock()
        mock__update_network.reset_mock()

        # Explicit network name is specified
        network_create_args = {"name": "network-name", "admin_state_up": False}

        scenario.create_and_update_networks(
            network_create_args=network_create_args,
            network_update_args=network_update_args)
        mock__create_network.assert_called_once_with(network_create_args)
        mock__update_network.assert_has_calls(
            [mock.call(mock__create_network.return_value,
                       network_update_args)])

    @mock.patch(NEUTRON_NETWORKS + "._delete_network")
    @mock.patch(NEUTRON_NETWORKS + "._create_network")
    def test_create_and_delete_networks(self, mock__create_network,
                                        mock__delete_network):
        neutron_scenario = network.NeutronNetworks(self.context)

        # Default options
        network_create_args = {}
        neutron_scenario.create_and_delete_networks()
        mock__create_network.assert_called_once_with(network_create_args)
        self.assertEqual(1, mock__delete_network.call_count)

        mock__create_network.reset_mock()
        mock__delete_network.reset_mock()

        # Explicit network name is specified
        network_create_args = {"name": "given-name"}
        neutron_scenario.create_and_delete_networks(
            network_create_args=network_create_args)
        mock__create_network.assert_called_once_with(network_create_args)
        self.assertEqual(1, mock__delete_network.call_count)

    def test_create_and_list_subnets(self):
        network_create_args = {"router:external": True}
        subnet_create_args = {"allocation_pools": []}
        subnet_cidr_start = "default_cidr"
        subnets_per_network = 5
        net = mock.MagicMock()

        scenario = network.NeutronNetworks(self.context)
        scenario._get_or_create_network = mock.Mock(return_value=net)
        scenario._create_subnets = mock.Mock()
        scenario._list_subnets = mock.Mock()

        scenario.create_and_list_subnets(
            network_create_args=network_create_args,
            subnet_create_args=subnet_create_args,
            subnet_cidr_start=subnet_cidr_start,
            subnets_per_network=subnets_per_network)

        scenario._get_or_create_network.assert_called_once_with(
            network_create_args)
        scenario._create_subnets.assert_called_once_with(
            net, subnet_create_args, subnet_cidr_start, subnets_per_network)

        scenario._list_subnets.assert_called_once_with()

    def test_create_and_update_subnets(self):
        network_create_args = {"router:external": True}
        subnet_create_args = {"allocation_pools": []}
        subnet_update_args = {"enabled_dhcp": True}
        subnet_cidr_start = "default_cidr"
        subnets_per_network = 5
        net = mock.MagicMock()
        subnets = [mock.MagicMock() for i in range(subnets_per_network)]

        scenario = network.NeutronNetworks(self.context)
        scenario._get_or_create_network = mock.Mock(return_value=net)
        scenario._create_subnets = mock.Mock(return_value=subnets)
        scenario._update_subnet = mock.Mock()

        scenario.create_and_update_subnets(
            subnet_update_args,
            network_create_args=network_create_args,
            subnet_create_args=subnet_create_args,
            subnet_cidr_start=subnet_cidr_start,
            subnets_per_network=subnets_per_network)

        scenario._get_or_create_network.assert_called_once_with(
            network_create_args)
        scenario._create_subnets.assert_called_once_with(
            net, subnet_create_args, subnet_cidr_start, subnets_per_network)
        scenario._update_subnet.assert_has_calls(
            [mock.call(s, subnet_update_args) for s in subnets])

    def test_create_and_delete_subnets(self):
        network_create_args = {"router:external": True}
        subnet_create_args = {"allocation_pools": []}
        subnet_cidr_start = "default_cidr"
        subnets_per_network = 5
        net = mock.MagicMock()
        subnets = [mock.MagicMock() for i in range(subnets_per_network)]

        scenario = network.NeutronNetworks(self.context)
        scenario._get_or_create_network = mock.Mock(return_value=net)
        scenario._create_subnets = mock.Mock(return_value=subnets)
        scenario._delete_subnet = mock.Mock()

        scenario.create_and_delete_subnets(
            network_create_args=network_create_args,
            subnet_create_args=subnet_create_args,
            subnet_cidr_start=subnet_cidr_start,
            subnets_per_network=subnets_per_network)

        scenario._get_or_create_network.assert_called_once_with(
            network_create_args)
        scenario._create_subnets.assert_called_once_with(
            net, subnet_create_args, subnet_cidr_start, subnets_per_network)
        scenario._delete_subnet.assert_has_calls(
            [mock.call(s) for s in subnets])

    def test_create_and_list_routers(self):
        network_create_args = {"router:external": True}
        subnet_create_args = {"allocation_pools": []}
        subnet_cidr_start = "default_cidr"
        subnets_per_network = 5
        router_create_args = {"admin_state_up": True}

        scenario = network.NeutronNetworks(self.context)
        scenario._create_network_structure = mock.Mock()
        scenario._list_routers = mock.Mock()

        scenario.create_and_list_routers(
            network_create_args=network_create_args,
            subnet_create_args=subnet_create_args,
            subnet_cidr_start=subnet_cidr_start,
            subnets_per_network=subnets_per_network,
            router_create_args=router_create_args)

        scenario._create_network_structure.assert_called_once_with(
            network_create_args, subnet_create_args, subnet_cidr_start,
            subnets_per_network, router_create_args)
        scenario._list_routers.assert_called_once_with()

    def test_create_and_update_routers(self):
        router_update_args = {"admin_state_up": False}
        network_create_args = {"router:external": True}
        subnet_create_args = {"allocation_pools": []}
        subnet_cidr_start = "default_cidr"
        subnets_per_network = 5
        router_create_args = {"admin_state_up": True}
        net = mock.MagicMock()
        subnets = [mock.MagicMock() for i in range(subnets_per_network)]
        routers = [mock.MagicMock() for i in range(subnets_per_network)]

        scenario = network.NeutronNetworks(self.context)
        scenario._create_network_structure = mock.Mock(
            return_value=(net, subnets, routers))
        scenario._update_router = mock.Mock()

        scenario.create_and_update_routers(
            router_update_args,
            network_create_args=network_create_args,
            subnet_create_args=subnet_create_args,
            subnet_cidr_start=subnet_cidr_start,
            subnets_per_network=subnets_per_network,
            router_create_args=router_create_args)

        scenario._create_network_structure.assert_called_once_with(
            network_create_args, subnet_create_args, subnet_cidr_start,
            subnets_per_network, router_create_args)

        update_calls = [mock.call(router, router_update_args)
                        for router in routers]
        scenario._update_router.assert_has_calls(update_calls)

    def test_create_and_delete_routers(self):
        network_create_args = {"router:external": True}
        subnet_create_args = {"allocation_pools": []}
        subnet_cidr_start = "default_cidr"
        subnets_per_network = 5
        router_create_args = {"admin_state_up": True}
        net = mock.MagicMock()
        subnets = [mock.MagicMock() for i in range(subnets_per_network)]
        routers = [mock.MagicMock() for i in range(subnets_per_network)]

        scenario = network.NeutronNetworks(self.context)
        scenario._create_network_structure = mock.Mock(
            return_value=(net, subnets, routers))
        scenario._remove_interface_router = mock.Mock()
        scenario._delete_router = mock.Mock()

        scenario.create_and_delete_routers(
            network_create_args=network_create_args,
            subnet_create_args=subnet_create_args,
            subnet_cidr_start=subnet_cidr_start,
            subnets_per_network=subnets_per_network,
            router_create_args=router_create_args)

        scenario._create_network_structure.assert_called_once_with(
            network_create_args, subnet_create_args, subnet_cidr_start,
            subnets_per_network, router_create_args)

        scenario._remove_interface_router.assert_has_calls([
            mock.call(subnets[i]["subnet"], routers[i]["router"])
            for i in range(subnets_per_network)])
        scenario._delete_router.assert_has_calls([mock.call(router)
                                                  for router in routers])

    def test_create_and_list_ports(self):
        port_create_args = {"allocation_pools": []}
        ports_per_network = 10
        network_create_args = {"router:external": True}
        net = mock.MagicMock()

        scenario = network.NeutronNetworks(self.context)
        scenario._get_or_create_network = mock.Mock(return_value=net)
        scenario._create_port = mock.MagicMock()
        scenario._list_ports = mock.Mock()

        scenario.create_and_list_ports(
            network_create_args=network_create_args,
            port_create_args=port_create_args,
            ports_per_network=ports_per_network)
        scenario._get_or_create_network.assert_called_once_with(
            network_create_args)
        scenario._create_port.assert_has_calls(
            [mock.call(net, port_create_args)
             for i in range(ports_per_network)])

        scenario._list_ports.assert_called_once_with()

    def test_create_and_update_ports(self):
        port_update_args = {"admin_state_up": False},
        port_create_args = {"allocation_pools": []}
        ports_per_network = 10
        network_create_args = {"router:external": True}
        net = mock.MagicMock()
        ports = [mock.MagicMock() for i in range(ports_per_network)]

        scenario = network.NeutronNetworks(self.context)
        scenario._get_or_create_network = mock.Mock(return_value=net)
        scenario._create_port = mock.Mock(side_effect=ports)
        scenario._update_port = mock.Mock()

        scenario.create_and_update_ports(
            port_update_args,
            network_create_args=network_create_args,
            port_create_args=port_create_args,
            ports_per_network=ports_per_network)
        scenario._get_or_create_network.assert_called_once_with(
            network_create_args)
        scenario._create_port.assert_has_calls(
            [mock.call(net, port_create_args)
             for i in range(ports_per_network)])
        scenario._update_port.assert_has_calls(
            [mock.call(p, port_update_args) for p in ports])

    def test_create_and_delete_ports(self):
        port_create_args = {"allocation_pools": []}
        ports_per_network = 10
        network_create_args = {"router:external": True}
        net = mock.MagicMock()
        ports = [mock.MagicMock() for i in range(ports_per_network)]

        scenario = network.NeutronNetworks(self.context)
        scenario._get_or_create_network = mock.Mock(return_value=net)
        scenario._create_port = mock.Mock(side_effect=ports)
        scenario._delete_port = mock.Mock()

        scenario.create_and_delete_ports(
            network_create_args=network_create_args,
            port_create_args=port_create_args,
            ports_per_network=ports_per_network)
        scenario._get_or_create_network.assert_called_once_with(
            network_create_args)
        scenario._create_port.assert_has_calls(
            [mock.call(net, port_create_args)
             for i in range(ports_per_network)])
        scenario._delete_port.assert_has_calls(
            [mock.call(p) for p in ports])

    @ddt.data(
        {"floating_network": "ext-net"},
        {"floating_network": "ext-net",
         "floating_ip_args": {"floating_ip_address": "1.1.1.1"}},
    )
    @ddt.unpack
    def test_create_and_list_floating_ips(self, floating_network=None,
                                          floating_ip_args=None):
        scenario = network.NeutronNetworks()
        floating_ip_args = floating_ip_args or {}
        scenario._create_floatingip = mock.Mock()
        scenario._list_floating_ips = mock.Mock()
        scenario.create_and_list_floating_ips(
            floating_network=floating_network,
            floating_ip_args=floating_ip_args)
        scenario._create_floatingip.assert_called_once_with(
            floating_network, **floating_ip_args)
        scenario._list_floating_ips.assert_called_once_with()

    @ddt.data(
        {"floating_network": "ext-net"},
        {"floating_network": "ext-net",
         "floating_ip_args": {"floating_ip_address": "1.1.1.1"}},
    )
    @ddt.unpack
    def test_create_and_delete_floating_ips(self, floating_network=None,
                                            floating_ip_args=None):
        scenario = network.NeutronNetworks()
        floating_ip_args = floating_ip_args or {}
        fip = {"floatingip": {"id": "floating-ip-id"}}
        scenario._create_floatingip = mock.Mock(return_value=fip)
        scenario._delete_floating_ip = mock.Mock()
        scenario.create_and_delete_floating_ips(
            floating_network=floating_network,
            floating_ip_args=floating_ip_args)
        scenario._create_floatingip.assert_called_once_with(
            floating_network, **floating_ip_args)
        scenario._delete_floating_ip.assert_called_once_with(
            scenario._create_floatingip.return_value["floatingip"])



# Copyright 2013: Mirantis Inc.
# All Rights Reserved.
#
#    Licensed under the Apache License, Version 2.0 (the "License"); you may
#    not use this file except in compliance with the License. You may obtain
#    a copy of the License at
#
#         http://www.apache.org/licenses/LICENSE-2.0
#
#    Unless required by applicable law or agreed to in writing, software
#    distributed under the License is distributed on an "AS IS" BASIS, WITHOUT
#    WARRANTIES OR CONDITIONS OF ANY KIND, either express or implied. See the
#    License for the specific language governing permissions and limitations
#    under the License.

import jsonschema
import mock
import netaddr

from rally.deployment.serverprovider.providers import lxc
from rally import exceptions
from tests.unit import test


MOD_NAME = "rally.deployment.serverprovider.providers.lxc."


class HelperFunctionsTestCase(test.TestCase):

    @mock.patch(MOD_NAME + "open", create=True, return_value="fake_script")
    def test__get_script(self, mock_open):
        script = lxc._get_script("script.sh")
        self.assertEqual("fake_script", script)
        path = mock_open.mock_calls[0][1][0]
        mode = mock_open.mock_calls[0][1][1]
        self.assertTrue(path.endswith(
            "rally/deployment/serverprovider/providers/lxc/script.sh"))
        self.assertEqual("rb", mode)

    @mock.patch(MOD_NAME + "_get_script", return_value="fake_script")
    @mock.patch(MOD_NAME + "moves.StringIO")
    def test__get_script_from_template(self, mock_string_io, mock__get_script):
        mock__get_script.return_value = fake_script = mock.Mock()
        fake_script.read.return_value = "fake_data {k1} {k2}"
        mock_string_io.return_value = "fake_formatted_script"
        script = lxc._get_script_from_template("fake_tpl", k1="v1", k2="v2")
        self.assertEqual("fake_formatted_script", script)
        mock_string_io.assert_called_once_with("fake_data v1 v2")


class LxcHostTestCase(test.TestCase):

    def setUp(self):
        super(LxcHostTestCase, self).setUp()

        sample_config = {"network": "10.1.1.0/24",
                         "forward_ssh": True,
                         "tunnel_to": ["1.1.1.1", "2.2.2.2"]}
        self.server = mock.Mock()
        self.server.host = "fake_server_ip"
        self.server.get_credentials.return_value = {"ip": "3.3.3.3"}
        self.host = lxc.LxcHost(self.server, sample_config)

    @mock.patch(MOD_NAME + "provider.Server")
    def test__get_updated_server(self, mock_server):
        server = self.host._get_updated_server(host="4.4.4.4")
        new_server = mock_server.from_credentials({"host": "4.4.4.4"})
        self.assertEqual(new_server, server)

    def test_backingstore_btrfs(self):
        self.server.ssh.execute.return_value = [0, "", ""]
        self.assertEqual("btrfs", self.host.backingstore)
        self.assertEqual("btrfs", self.host.backingstore)
        # second call will return cached value
        self.assertEqual([mock.call.ssh.execute("df -t btrfs /var/lib/lxc/")],
                         self.server.mock_calls)

    def test_backingstore_none(self):
        self.server.ssh.execute.return_value = [-1, "", ""]
        self.assertEqual("", self.host.backingstore)

    @mock.patch(MOD_NAME + "moves.StringIO")
    @mock.patch(MOD_NAME + "_get_script", return_value="fake_script")
    def test_prepare(self, mock__get_script, mock_string_io):
        mock_string_io.return_value = fake_conf = mock.Mock()
        self.host.create_local_tunnels = mock.Mock()
        self.host.create_remote_tunnels = mock.Mock()

        self.host.prepare()

        write_calls = [
            mock.call("LXC_DHCP_MAX=\"253\"\n"),
            mock.call("LXC_NETMASK=\"255.255.255.0\"\n"),
            mock.call("LXC_ADDR=\"10.1.1.1\"\n"),
            mock.call("LXC_DHCP_RANGE=\"10.1.1.2,10.1.1.254\"\n"),
            mock.call("LXC_NETWORK=\"10.1.1.0/24\"\n"),
            mock.call("LXC_BRIDGE=\"lxcbr0\"\n"),
            mock.call("USE_LXC_BRIDGE=\"true\"\n")
        ]
        fake_conf.write.assert_has_calls(write_calls, any_order=True)
        ssh_calls = [mock.call.run("cat > /tmp/.lxc_default", stdin=fake_conf),
                     mock.call.run("/bin/sh", stdin="fake_script")]
        self.assertEqual(ssh_calls, self.server.ssh.mock_calls)
        self.host.create_local_tunnels.assert_called_once_with()
        self.host.create_remote_tunnels.assert_called_once_with()

    @mock.patch(MOD_NAME + "os.unlink")
    @mock.patch(MOD_NAME + "_get_script_from_template")
    def test_create_local_tunnels(self, mock__get_script_from_template,
                                  mock_unlink):
        mock__get_script_from_template.side_effect = ["s1", "s2"]
        self.host.create_local_tunnels()
        getscript_calls = [
            mock.call("tunnel-local.sh", local="fake_server_ip",
                      net=netaddr.IPNetwork("10.1.1.0/24"), remote="1.1.1.1"),
            mock.call("tunnel-local.sh", local="fake_server_ip",
                      net=netaddr.IPNetwork("10.1.1.0/24"), remote="2.2.2.2"),
        ]
        self.assertEqual(
            getscript_calls, mock__get_script_from_template.mock_calls)
        self.assertEqual([mock.call("/bin/sh", stdin="s1"),
                          mock.call("/bin/sh", stdin="s2")],
                         self.server.ssh.run.mock_calls)

    @mock.patch(MOD_NAME + "_get_script_from_template")
    def test_create_remote_tunnels(self, mock__get_script_from_template):
        mock__get_script_from_template.side_effect = ["s1", "s2"]
        fake_server = mock.Mock()
        self.host._get_updated_server = mock.Mock(return_value=fake_server)
        self.host.create_remote_tunnels()
        self.assertEqual([mock.call("/bin/sh", stdin="s1"),
                          mock.call("/bin/sh", stdin="s2")],
                         fake_server.ssh.run.mock_calls)

    def test_delete_tunnels(self):
        s1 = mock.Mock()
        s2 = mock.Mock()
        self.host._get_updated_server = mock.Mock(side_effect=[s1, s2])

        self.host.delete_tunnels()
        s1.ssh.execute.assert_called_once_with("ip tun del t10.1.1.0")
        s2.ssh.execute.assert_called_once_with("ip tun del t10.1.1.0")
        self.assertEqual([mock.call("ip tun del t1.1.1.1"),
                          mock.call("ip tun del t2.2.2.2")],
                         self.server.ssh.execute.mock_calls)

    @mock.patch(MOD_NAME + "time.sleep")
    def test_get_ip(self, mock_sleep):
        s1 = "link/ether fe:54:00:d3:f5:98 brd ff:ff:ff:ff:ff:ff"
        s2 = s1 + "\n   inet 10.20.0.1/24 scope global br1"
        self.host.server.ssh.execute.side_effect = [(0, s1, ""), (0, s2, "")]
        ip = self.host.get_ip("name")
        self.assertEqual("10.20.0.1", ip)
        self.assertEqual([mock.call("lxc-attach -n name ip"
                                    " addr list dev eth0")] * 2,
                         self.host.server.ssh.execute.mock_calls)

    def test_create_container(self):
        self.host.configure_container = mock.Mock()
        self.host._backingstore = "btrfs"
        self.host.create_container("name", "dist")
        self.server.ssh.run.assert_called_once_with(
            "lxc-create -B btrfs -n name -t dist")
        self.assertEqual(["name"], self.host.containers)
        self.host.configure_container.assert_called_once_with("name")

        # check with no btrfs
        self.host._backingstore = ""
        self.host.create_container("name", "dist")
        self.assertEqual(mock.call("lxc-create -n name -t dist"),
                         self.server.ssh.run.mock_calls[1])

        # check release
        self.host.create_container("name", "ubuntu", "raring")
        self.host.create_container("name", "debian", "woody")
        expected = [mock.call("lxc-create -n name -t ubuntu -- -r raring"),
                    mock.call("SUITE=woody lxc-create -n name -t debian")]
        self.assertEqual(expected, self.server.ssh.run.mock_calls[2:])

    def test_create_clone(self):
        self.host._backingstore = "btrfs"
        self.host.configure_container = mock.Mock()
        self.host.create_clone("name", "src")
        self.server.ssh.execute.assert_called_once_with("lxc-clone --snapshot"
                                                        " -o src -n name")
        self.assertEqual(["name"], self.host.containers)

        # check with no btrfs
        self.host._backingstore = ""
        self.host.create_clone("name", "src")
        self.assertEqual(mock.call("lxc-clone -o src -n name"),
                         self.server.ssh.execute.mock_calls[1])

    @mock.patch(MOD_NAME + "os.path.join")
    @mock.patch(MOD_NAME + "_get_script")
    def test_configure_container(self, mock__get_script, mock_join):
        mock__get_script.return_value = "fake_script"
        mock_join.return_value = "fake_path"
        self.server.ssh.execute.return_value = 0, "", ""
        self.host.configure_container("name")
        self.server.ssh.run.assert_called_once_with(
            "/bin/sh -e -s fake_path", stdin="fake_script")

    def test_start_containers(self):
        self.host.containers = ["c1", "c2"]
        self.host.start_containers()
        calls = [mock.call("lxc-start -d -n c1"),
                 mock.call("lxc-start -d -n c2")]
        self.assertEqual(calls, self.server.ssh.run.mock_calls)

    def test_stop_containers(self):
        self.host.containers = ["c1", "c2"]
        self.host.stop_containers()
        calls = [
            mock.call("lxc-stop -n c1"),
            mock.call("lxc-stop -n c2"),
        ]
        self.assertEqual(calls, self.server.ssh.run.mock_calls)

    def test_destroy_containers(self):
        self.host.containers = ["c1", "c2"]
        self.host.destroy_containers()
        calls = [
            mock.call("lxc-stop -n c1"), mock.call("lxc-destroy -n c1"),
            mock.call("lxc-stop -n c2"), mock.call("lxc-destroy -n c2"),
        ]
        self.assertEqual(calls, self.server.ssh.run.mock_calls)

    def test_get_server_object(self):
        fake_server = mock.Mock()
        self.host.get_ip = mock.Mock(return_value="ip")
        self.host.get_port = mock.Mock(return_value=42)
        self.host._get_updated_server = mock.Mock(return_value=fake_server)

        so = self.host.get_server_object("c1", wait=False)

        self.assertEqual(fake_server, so)
        self.host.get_port.assert_called_once_with("ip")
        self.host._get_updated_server.assert_called_once_with(port=42)
        self.assertFalse(fake_server.ssh.wait.mock_calls)
        so = self.host.get_server_object("c1", wait=True)
        fake_server.ssh.wait.assert_called_once_with(timeout=300)

    @mock.patch(MOD_NAME + "LxcHost.get_server_object")
    def test_get_server_objects(self, mock_get_server_object):
        mock_get_server_object.side_effect = ["s1", "s2"]
        self.host.containers = ["c1", "c2"]
        retval = list(self.host.get_server_objects(wait="wait"))
        self.assertEqual(["s1", "s2"], retval)
        self.assertEqual([mock.call("c1", "wait"), mock.call("c2", "wait")],
                         mock_get_server_object.mock_calls)


class LxcProviderTestCase(test.TestCase):

    def setUp(self):
        super(LxcProviderTestCase, self).setUp()
        self.config = {
            "type": "LxcProvider",
            "distribution": "ubuntu",
            "start_lxc_network": "10.1.1.0/29",
            "containers_per_host": 2,
            "tunnel_to": ["10.10.10.10", "20.20.20.20"],
            "container_name_prefix": "rally-lxc",
            "host_provider": {
                "name": "ExistingServers",
                "credentials": [{"user": "root", "host": "host1.net"},
                                {"user": "root", "host": "host2.net"}]}
        }
        self.deployment = {"uuid": "fake_uuid"}
        self.provider = lxc.LxcProvider(self.deployment, self.config)

    def test_validate(self):
        self.provider.validate()

    def test_validate_invalid_tunnel(self):
        config = self.config.copy()
        config["tunnel_to"] = "ok"
        self.assertRaises(jsonschema.ValidationError,
                          lxc.LxcProvider, self.deployment, config)

    def test_validate_required_field(self):
        config = self.config.copy()
        del(config["host_provider"])
        self.assertRaises(jsonschema.ValidationError,
                          lxc.LxcProvider, self.deployment, config)

    def test_validate_too_small_network(self):
        config = self.config.copy()
        config["containers_per_host"] = 42
        self.assertRaises(exceptions.InvalidConfigException,
                          lxc.LxcProvider, self.deployment, config)

    @mock.patch(MOD_NAME + "LxcHost")
    @mock.patch(MOD_NAME + "provider.ProviderFactory.get_provider")
    def test_create_servers(self, mock_provider_factory_get_provider,
                            mock_lxc_host):
        fake_provider = mock.Mock()
        fake_provider.create_servers.return_value = ["server1", "server2"]
        fake_hosts = []
        fake_sos = []
        for i in (1, 2):
            fake_host_sos = [mock.Mock(), mock.Mock()]
            fake_sos.extend(fake_host_sos)
            fake_host = mock.Mock()
            fake_host.containers = ["c-%d-1" % i, "c-%d-2" % i]
            fake_host._port_cache = {1: i, 2: i}
            fake_host.config = {"network": "fake-%d" % i}
            fake_host.server.get_credentials.return_value = {"ip": "f%d" % i}
            fake_host.get_server_objects.return_value = fake_host_sos
            fake_hosts.append(fake_host)
        mock_lxc_host.side_effect = fake_hosts
        mock_provider_factory_get_provider.return_value = fake_provider

        fake_info = [
            {"host": {"ip": "f1"},
             "config": {"network": "fake-1"},
             "forwarded_ports": [(1, 1), (2, 1)],
             "container_names": ["c-1-1", "c-1-2"]},
            {"host": {"ip": "f2"},
             "config": {"network": "fake-2"},
             "forwarded_ports": [(1, 2), (2, 2)],
             "container_names": ["c-2-1", "c-2-2"]}]

        def res_create(actual_info):
            expected_info = fake_info.pop(0)
            self.assertEqual(expected_info["host"], actual_info["host"])
            self.assertEqual(expected_info["config"], actual_info["config"])
            self.assertEqual(expected_info["container_names"],
                             actual_info["container_names"])
            self.assertSequenceEqual(expected_info["forwarded_ports"],
                                     actual_info["forwarded_ports"])

        fake_res = mock.MagicMock()
        fake_res.create = res_create

        with mock.patch.object(self.provider, "resources", fake_res):
            servers = self.provider.create_servers()

        self.assertEqual(fake_sos, servers)

        host1_calls = [
            mock.call.prepare(),
            mock.call.create_container("rally-lxc-000-10-1-1-0",
                                       "ubuntu", None),
            mock.call.create_clone("rally-lxc-001-10-1-1-0",
                                   "rally-lxc-000-10-1-1-0"),
            mock.call.start_containers(),
            mock.call.get_server_objects(),
            mock.call.server.get_credentials(),
        ]
        host2_calls = [
            mock.call.prepare(),
            mock.call.create_container("rally-lxc-000-10-1-1-8",
                                       "ubuntu", None),
            mock.call.create_clone("rally-lxc-001-10-1-1-8",
                                   "rally-lxc-000-10-1-1-8"),
            mock.call.start_containers(),
            mock.call.get_server_objects(),
            mock.call.server.get_credentials(),
        ]

        self.assertEqual(host1_calls, fake_hosts[0].mock_calls)
        self.assertEqual(host2_calls, fake_hosts[1].mock_calls)

    @mock.patch(MOD_NAME + "LxcHost")
    @mock.patch(MOD_NAME + "provider.Server.from_credentials")
    def test_destroy_servers(self, mock_server_from_credentials,
                             mock_lxc_host):
        fake_resource = {"info": {"config": "fake_config",
                                  "host": "fake_credentials",
                                  "forwarded_ports": [1, 2],
                                  "container_names": ["n1", "n2"]}}
        fake_resource["id"] = "fake_res_id"
        fake_host = mock.Mock()
        mock_server_from_credentials.return_value = "fake_server"
        mock_lxc_host.return_value = fake_host
        self.provider.resources = mock.Mock()
        self.provider.resources.get_all.return_value = [fake_resource]

        with mock.patch.object(self.provider, "get_host_provider") as ghp:
            ghp.return_value = fake_host_provider = mock.Mock()
            self.provider.destroy_servers()

        mock_lxc_host.assert_called_once_with("fake_server", "fake_config")
        host_calls = [mock.call.destroy_containers(),
                      mock.call.destroy_ports([1, 2]),
                      mock.call.delete_tunnels()]
        self.assertEqual(host_calls, fake_host.mock_calls)
        self.provider.resources.delete.assert_called_once_with("fake_res_id")
        fake_host_provider.destroy_servers.assert_called_once_with()



# All Rights Reserved.
#
#    Licensed under the Apache License, Version 2.0 (the "License"); you may
#    not use this file except in compliance with the License. You may obtain
#    a copy of the License at
#
#         http://www.apache.org/licenses/LICENSE-2.0
#
#    Unless required by applicable law or agreed to in writing, software
#    distributed under the License is distributed on an "AS IS" BASIS, WITHOUT
#    WARRANTIES OR CONDITIONS OF ANY KIND, either express or implied. See the
#    License for the specific language governing permissions and limitations
#    under the License.

import abc
import bisect
import math

import six

from rally.common import costilius
from rally.common.plugin import plugin
from rally.common import streaming_algorithms as streaming
from rally.task.processing import utils


@six.add_metaclass(abc.ABCMeta)
@plugin.configure(name="base_output_chart")
class Chart(plugin.Plugin):
    """Base class for charts."""

    @abc.abstractproperty
    def widget(self):
        """Widget name to display this chart by JavaScript."""

    def __init__(self, workload_info, zipped_size=1000):
        """Setup initial values.

        :param workload_info: dict, generalized info about iterations.
                               The most important value is `iterations_count'
                               that should have int value of total data size
        :param zipped_size: int maximum number of points on scale
        """
        self._data = costilius.OrderedDict()  # Container for results
        self._workload_info = workload_info
        self.base_size = workload_info.get("iterations_count", 0)
        self.zipped_size = zipped_size

    def add_iteration(self, iteration):
        """Add iteration data.

        This method must be called for each iteration.
        If overridden, this method must use streaming data processing,
        so chart instance could process unlimited number of iterations,
        with low memory usage.
        """
        for name, value in self._map_iteration_values(iteration):
            if name not in self._data:
                self._data[name] = utils.GraphZipper(self.base_size,
                                                     self.zipped_size)
            self._data[name].add_point(value)

    def render(self):
        """Generate chart data ready for drawing."""
        return [(name, points.get_zipped_graph())
                for name, points in self._data.items()]

    def _fix_atomic_actions(self, iteration):
        """Set `0' for missed atomic actions.

        Since some atomic actions can absent in some iterations
        due to failures, this method must be used in all cases
        related to atomic actions processing.
        """
        for name in self._workload_info["atomic"]:
            iteration["atomic_actions"].setdefault(name, 0)
        return iteration

    @abc.abstractmethod
    def _map_iteration_values(self, iteration):
        """Get values for processing, from given iteration."""


class MainStackedAreaChart(Chart):

    widget = "StackedArea"

    def _map_iteration_values(self, iteration):
        if iteration["error"]:
            result = [("duration", 0), ("idle_duration", 0)]
            if self._workload_info["iterations_failed"]:
                result.append(
                    ("failed_duration",
                     iteration["duration"] + iteration["idle_duration"]))
        else:
            result = [("duration", iteration["duration"]),
                      ("idle_duration", iteration["idle_duration"])]
            if self._workload_info["iterations_failed"]:
                result.append(("failed_duration", 0))
        return result


class AtomicStackedAreaChart(Chart):

    widget = "StackedArea"

    def _map_iteration_values(self, iteration):
        iteration = self._fix_atomic_actions(iteration)
        atomics = list(iteration["atomic_actions"].items())
        if self._workload_info["iterations_failed"]:
            if iteration["error"]:
                failed_duration = (
                    iteration["duration"] + iteration["idle_duration"]
                    - sum([(a[1] or 0) for a in atomics]))
            else:
                failed_duration = 0
            atomics.append(("failed_duration", failed_duration))
        return atomics


class AvgChart(Chart):
    """Base class for charts with average results."""

    widget = "Pie"

    def add_iteration(self, iteration):
        for name, value in self._map_iteration_values(iteration):
            if name not in self._data:
                self._data[name] = streaming.MeanComputation()
            self._data[name].add(value or 0)

    def render(self):
        return [(k, v.result()) for k, v in self._data.items()]


class AtomicAvgChart(AvgChart):

    def _map_iteration_values(self, iteration):
        iteration = self._fix_atomic_actions(iteration)
        return list(iteration["atomic_actions"].items())


class LoadProfileChart(Chart):
    """Chart for parallel durations."""

    widget = "StackedArea"

    def __init__(self, workload_info, name="parallel iterations",
                 scale=200):
        """Setup chart with graph name and scale.

        :workload_info:  dict, generalized info about iterations
        :param name: str name for X axis
        :param scale: int number of X points
        """
        super(LoadProfileChart, self).__init__(workload_info)
        self._name = name
        self._duration = workload_info["load_duration"]
        self._tstamp_start = workload_info["tstamp_start"]

        # NOTE(amaretskiy): Determine a chart `step' - duration between
        #   two X points, rounded with minimal accuracy (digits after point)
        #   to improve JavaScript drawing performance.
        # Examples:
        #   scale  duration       step (initial)    accuracy  step
        #   200    30.8043010235  0.154021505117    1         0.2
        #   200    1.25884699821  0.00629423499107  3         0.006
        step = self._duration / float(scale)
        if step == 0:
            accuracy = 0
        else:
            accuracy = max(-int(math.floor(math.log10(step))), 0)
        step = round(step, accuracy)
        self._time_axis = [step * x
                           for x in six.moves.range(1, int(scale))
                           if (step * x) < self._duration]
        self._time_axis.append(self._duration)
        self._started = [0] * len(self._time_axis)
        self._stopped = [0] * len(self._time_axis)

    def _map_iteration_values(self, iteration):
        return (iteration["timestamp"],
                0 if iteration["error"] else iteration["duration"])

    def add_iteration(self, iteration):
        timestamp, duration = self._map_iteration_values(iteration)
        ts_start = timestamp - self._tstamp_start
        ts_stop = ts_start + duration
        self._started[bisect.bisect(self._time_axis, ts_start)] += 1
        self._stopped[bisect.bisect(self._time_axis, ts_stop)] += 1

    def render(self):
        data = []
        running = 0
        for ts, started, ended in zip(self._time_axis,
                                      self._started, self._stopped):
            running += started
            data.append([ts, running])
            running -= ended
        return [(self._name, data)]


class HistogramChart(Chart):
    """Base class for chart with histograms.

    This chart is relatively complex, because actually it is a set
    of histograms, that usually can be switched by dropdown select.
    And each histogram has several data views.
    """

    widget = "Histogram"

    def _init_views(self, min_value, max_value):
        """Generate initial data for each histogram view."""
        if not self.base_size:
            return []
        min_value, max_value = min_value or 0, max_value or 0
        views = []
        for view, bins in [
                ("Square Root Choice",
                 int(math.ceil(math.sqrt(self.base_size)))),
                ("Sturges Formula",
                 int(math.ceil(math.log(self.base_size, 2) + 1))),
                ("Rice Rule",
                 int(math.ceil(2 * self.base_size ** (1.0 / 3))))]:
            bin_width = float(max_value - min_value) / bins
            x_axis = [min_value + (bin_width * x) for x in range(1, bins + 1)]
            views.append({"view": view, "bins": bins,
                          "x": x_axis, "y": [0] * len(x_axis)})
        return views

    def add_iteration(self, iteration):
        for name, value in self._map_iteration_values(iteration):
            if name not in self._data:
                raise KeyError("Unexpected histogram name: %s" % name)
            for i, view in enumerate(self._data[name]["views"]):
                for bin_i, bin_v in enumerate(view["x"]):
                    if (value or 0) <= bin_v:
                        self._data[name]["views"][i]["y"][bin_i] += 1
                        break

    def render(self):
        data = []
        for name, hist in self._data.items():
            for idx, v in enumerate(hist["views"]):
                graph = {"key": name,
                         "view": v["view"],
                         "disabled": hist["disabled"],
                         "values": [{"x": x, "y": y}
                                    for x, y in zip(v["x"], v["y"])]}
                try:
                    data[idx].append(graph)
                except IndexError:
                    data.append([graph])
        return {"data": data, "views": [{"id": i, "name": d[0]["view"]}
                                        for i, d in enumerate(data)]}


class MainHistogramChart(HistogramChart):

    def __init__(self, workload_info):
        super(MainHistogramChart, self).__init__(workload_info)
        views = self._init_views(self._workload_info["min_duration"],
                                 self._workload_info["max_duration"])
        self._data["task"] = {"views": views, "disabled": None}

    def _map_iteration_values(self, iteration):
        return [("task", 0 if iteration["error"] else iteration["duration"])]


class AtomicHistogramChart(HistogramChart):

    def __init__(self, workload_info):
        super(AtomicHistogramChart, self).__init__(workload_info)
        for i, atomic in enumerate(self._workload_info["atomic"].items()):
            name, value = atomic
            self._data[name] = {
                "views": self._init_views(value["min_duration"],
                                          value["max_duration"]),
                "disabled": i}

    def _map_iteration_values(self, iteration):
        iteration = self._fix_atomic_actions(iteration)
        return list(iteration["atomic_actions"].items())


@six.add_metaclass(abc.ABCMeta)
class Table(Chart):
    """Base class for tables.

    Each Table subclass represents HTML table which can be easily rendered in
    report. Subclasses are responsible for setting up both columns and rows:
    columns are set simply by `columns' property (list of str columns names)
    and rows must be initialized in _data property, with the following format:
        self._data = {name: [streaming_ins, postprocess_func or None], ...}
          where:
            name - str name of table row parameter
            streaming_ins - instance of streaming algorithm
            postprocess_func - optional function that processes final result,
                               None means usage of default self._round()
        This can be done in __init__() or even in add_iteration().
    """

    widget = "Table"

    @abc.abstractproperty
    def columns(self):
        """List of columns names."""

    def _round(self, ins, has_result):
        """This is a default post-process function for table cell value.

        :param ins: streaming_algorithms.StreamingAlgorithm subclass instance
        :param has_result: bool, whether current row is effective
        :returns: rounded float
        :returns: str "n/a"
        """
        return round(ins.result(), 3) if has_result else "n/a"

    def _row_has_results(self, values):
        """Determine whether row can be assumed as having values.

        :param values: row values list
                       [(StreamingAlgorithm, function or None), ...]
        :returns: bool
        """
        for ins, fn in values:
            if isinstance(ins, streaming.MinComputation):
                return bool(ins.result())
        return True

    def get_rows(self):
        """Collect rows values finally, after all data is processed.

        :returns: [str_name, (float or str), (float or str), ...]
        """
        rows = []
        for name, values in self._data.items():
            row = [name]
            has_result = self._row_has_results(values)
            for ins, fn in values:
                fn = fn or self._round
                row.append(fn(ins, has_result))
            rows.append(row)
        return rows

    def render(self):
        return {"cols": self.columns, "rows": self.get_rows()}


class MainStatsTable(Table):

    columns = ["Action", "Min (sec)", "Median (sec)", "90%ile (sec)",
               "95%ile (sec)", "Max (sec)", "Avg (sec)", "Success", "Count"]

    def __init__(self, *args, **kwargs):
        super(MainStatsTable, self).__init__(*args, **kwargs)
        iters_num = self._workload_info["iterations_count"]
        for name in (list(self._workload_info["atomic"].keys()) + ["total"]):
            self._data[name] = [
                [streaming.MinComputation(), None],
                [streaming.PercentileComputation(0.5, iters_num), None],
                [streaming.PercentileComputation(0.9, iters_num), None],
                [streaming.PercentileComputation(0.95, iters_num), None],
                [streaming.MaxComputation(), None],
                [streaming.MeanComputation(), None],
                [streaming.MeanComputation(),
                 lambda st, has_result: ("%.1f%%" % (st.result() * 100)
                                         if has_result else "n/a")],
                [streaming.IncrementComputation(),
                 lambda st, has_result: st.result()]]

    def _map_iteration_values(self, iteration):
        return dict(iteration["atomic_actions"], total=iteration["duration"])

    def add_iteration(self, iteration):
        for name, value in self._map_iteration_values(iteration).items():
            self._data[name][-1][0].add()
            if iteration["error"]:
                self._data[name][-2][0].add(0)
            else:
                self._data[name][-2][0].add(1)
                for idx, dummy in enumerate(self._data[name][:-2]):
                    self._data[name][idx][0].add(value)


class OutputChart(Chart):
    """Base class for charts related to scenario output."""

    def __init__(self, workload_info,
                 zipped_size=1000, title="", description=""):
        super(OutputChart, self).__init__(workload_info, zipped_size)
        self.title = title
        self.description = description

    def _map_iteration_values(self, iteration):
        return iteration

    def render(self):
        return {"title": self.title,
                "description": self.description,
                "widget": self.widget,
                "data": super(OutputChart, self).render()}


@plugin.configure(name="StackedArea")
class OutputStackedAreaChart(OutputChart):

    widget = "StackedArea"


@plugin.configure(name="Pie")
class OutputAvgChart(OutputChart, AvgChart):

    widget = "Pie"


@plugin.configure(name="Table")
class OutputTable(OutputChart, Table):

    widget = "Table"


@plugin.configure(name="StatsTable")
class OutputStatsTable(OutputTable):

    columns = ["Action", "Min (sec)", "Median (sec)", "90%ile (sec)",
               "95%ile (sec)", "Max (sec)", "Avg (sec)", "Count"]

    def add_iteration(self, iteration):
        for name, value in self._map_iteration_values(iteration):
            if name not in self._data:
                iters_num = self._workload_info["iterations_count"]
                self._data[name] = [
                    [streaming.MinComputation(), None],
                    [streaming.PercentileComputation(0.5, iters_num), None],
                    [streaming.PercentileComputation(0.9, iters_num), None],
                    [streaming.PercentileComputation(0.95, iters_num), None],
                    [streaming.MaxComputation(), None],
                    [streaming.MeanComputation(), None],
                    [streaming.IncrementComputation(),
                     lambda v, na: v.result()]]

            self._data[name][-1][0].add(None)
            self._data[name][-2][0].add(1)
            for idx, dummy in enumerate(self._data[name][:-1]):
                self._data[name][idx][0].add(value)



# All Rights Reserved.
#
#    Licensed under the Apache License, Version 2.0 (the "License"); you may
#    not use this file except in compliance with the License. You may obtain
#    a copy of the License at
#
#         http://www.apache.org/licenses/LICENSE-2.0
#
#    Unless required by applicable law or agreed to in writing, software
#    distributed under the License is distributed on an "AS IS" BASIS, WITHOUT
#    WARRANTIES OR CONDITIONS OF ANY KIND, either express or implied. See the
#    License for the specific language governing permissions and limitations
#    under the License.

import mock

from rally.plugins.openstack.context.quotas import nova_quotas as quotas
from tests.unit import test


class NovaQuotasTestCase(test.TestCase):

    @mock.patch("rally.plugins.openstack.context."
                "quotas.quotas.osclients.Clients")
    def test_update(self, mock_clients):
        nova_quotas = quotas.NovaQuotas(mock_clients)
        tenant_id = mock.MagicMock()
        quotas_values = {
            "instances": 10,
            "cores": 100,
            "ram": 100000,
            "floating_ips": 100,
            "fixed_ips": 10000,
            "metadata_items": 5,
            "injected_files": 5,
            "injected_file_content_bytes": 2048,
            "injected_file_path_bytes": 1024,
            "key_pairs": 50,
            "security_groups": 50,
            "security_group_rules": 50
        }
        nova_quotas.update(tenant_id, **quotas_values)
        mock_clients.nova().quotas.update.assert_called_once_with(
            tenant_id, **quotas_values)

    @mock.patch("rally.plugins.openstack.context."
                "quotas.quotas.osclients.Clients")
    def test_delete(self, mock_clients):
        nova_quotas = quotas.NovaQuotas(mock_clients)
        tenant_id = mock.MagicMock()
        nova_quotas.delete(tenant_id)
        mock_clients.nova().quotas.delete.assert_called_once_with(tenant_id)



# All Rights Reserved.
#
#    Licensed under the Apache License, Version 2.0 (the "License"); you may
#    not use this file except in compliance with the License. You may obtain
#    a copy of the License at
#
#         http://www.apache.org/licenses/LICENSE-2.0
#
#    Unless required by applicable law or agreed to in writing, software
#    distributed under the License is distributed on an "AS IS" BASIS, WITHOUT
#    WARRANTIES OR CONDITIONS OF ANY KIND, either express or implied. See the
#    License for the specific language governing permissions and limitations
#    under the License.

import mock

from rally.plugins.openstack.scenarios.fuel import environments
from tests.unit import test


class FuelEnvironmentsTestCase(test.ScenarioTestCase):

    def test_create_and_list_environments(self):
        scenario = environments.FuelEnvironments(self.context)

        scenario._create_environment = mock.Mock()
        scenario._list_environments = mock.Mock()

        scenario.create_and_list_environments(
            release_id=2, network_provider="test_neutron",
            deployment_mode="test_mode", net_segment_type="test_type")
        scenario._create_environment.assert_called_once_with(
            release_id=2, network_provider="test_neutron",
            deployment_mode="test_mode", net_segment_type="test_type")
        scenario._list_environments.assert_called_once_with()

    def test_create_and_delete_environments(self):
        scenario = environments.FuelEnvironments()

        scenario._create_environment = mock.Mock(return_value=42)
        scenario._delete_environment = mock.Mock()

        scenario.create_and_delete_environment(
            release_id=2, network_provider="test_neutron",
            deployment_mode="test_mode", net_segment_type="test_type")

        scenario._create_environment.assert_called_once_with(
            release_id=2, network_provider="test_neutron",
            deployment_mode="test_mode", net_segment_type="test_type")
        scenario._delete_environment.assert_called_once_with(42, 5)



#    Licensed under the Apache License, Version 2.0 (the "License"); you may
#    not use this file except in compliance with the License. You may obtain
#    a copy of the License at
#
#         http://www.apache.org/licenses/LICENSE-2.0
#
#    Unless required by applicable law or agreed to in writing, software
#    distributed under the License is distributed on an "AS IS" BASIS, WITHOUT
#    WARRANTIES OR CONDITIONS OF ANY KIND, either express or implied. See the
#    License for the specific language governing permissions and limitations
#    under the License.

import copy

import jsonschema
import mock

from rally.common import utils
from rally import exceptions
from rally.plugins.openstack.context.keystone import api_versions
from tests.unit import test


class OpenStackServicesTestCase(test.TestCase):

    def setUp(self):
        super(OpenStackServicesTestCase, self).setUp()
        self.mock_clients = mock.patch("rally.osclients.Clients").start()
        self.mock_kc = self.mock_clients.return_value.keystone.return_value
        self.mock_kc.service_catalog.get_endpoints.return_value = []
        self.mock_kc.services.list.return_value = []

    def test_validate_correct_config(self):
        api_versions.OpenStackAPIVersions.validate({
            "nova": {"service_type": "compute", "version": 2},
            "cinder": {"service_name": "cinderv2", "version": 2},
            "neutron": {"service_type": "network"},
            "glance": {"service_name": "glance"},
            "heat": {"version": 1}
        })

    def test_validate_wrong_configs(self):
        self.assertRaises(
            exceptions.PluginNotFound,
            api_versions.OpenStackAPIVersions.validate,
            {"invalid": {"service_type": "some_type"}},
            "Non-existing clients should be caught.")

        self.assertRaises(
            jsonschema.ValidationError,
            api_versions.OpenStackAPIVersions.validate,
            {"nova": {"some_key": "some_value"}},
            "Additional properties should be restricted.")

        self.assertRaises(
            exceptions.ValidationError,
            api_versions.OpenStackAPIVersions.validate,
            {"keystone": {"service_type": "identity"}},
            "Setting service_type is allowed only for those clients, which "
            "support it.")

        self.assertRaises(
            exceptions.ValidationError,
            api_versions.OpenStackAPIVersions.validate,
            {"keystone": {"service_name": "keystone"}},
            "Setting service_name is allowed only for those clients, which "
            "support it.")

        self.assertRaises(
            exceptions.ValidationError,
            api_versions.OpenStackAPIVersions.validate,
            {"keystone": {"version": 1}},
            "Setting version is allowed only for those clients, which "
            "support it.")

        self.assertRaises(
            exceptions.ValidationError,
            api_versions.OpenStackAPIVersions.validate,
            {"nova": {"version": 666}},
            "Unsupported version should be caught.")

    def test_setup_with_wrong_service_name(self):
        context = {
            "config": {api_versions.OpenStackAPIVersions.get_name(): {
                "nova": {"service_name": "service_name"}}},
            "admin": {"credential": mock.MagicMock()},
            "users": [{"credential": mock.MagicMock()}]}
        ctx = api_versions.OpenStackAPIVersions(context)
        self.assertRaises(exceptions.ValidationError, ctx.setup)
        self.mock_kc.service_catalog.get_endpoints.assert_called_once_with()
        self.mock_kc.services.list.assert_called_once_with()

    def test_setup_with_wrong_service_name_and_without_admin(self):
        context = {
            "config": {api_versions.OpenStackAPIVersions.get_name(): {
                "nova": {"service_name": "service_name"}}},
            "users": [{"credential": mock.MagicMock()}]}
        ctx = api_versions.OpenStackAPIVersions(context)
        self.assertRaises(exceptions.BenchmarkSetupFailure, ctx.setup)
        self.mock_kc.service_catalog.get_endpoints.assert_called_once_with()
        self.assertFalse(self.mock_kc.services.list.called)

    def test_setup_with_wrong_service_type(self):
        context = {
            "config": {api_versions.OpenStackAPIVersions.get_name(): {
                "nova": {"service_type": "service_type"}}},
            "users": [{"credential": mock.MagicMock()}]}
        ctx = api_versions.OpenStackAPIVersions(context)
        self.assertRaises(exceptions.ValidationError, ctx.setup)
        self.mock_kc.service_catalog.get_endpoints.assert_called_once_with()

    def test_setup_with_service_name(self):
        self.mock_kc.services.list.return_value = [
            utils.Struct(type="computev21", name="NovaV21")]
        context = {
            "config": {api_versions.OpenStackAPIVersions.get_name(): {
                "nova": {"service_name": "NovaV21"}}},
            "admin": {"credential": mock.MagicMock()},
            "users": [{"credential": mock.MagicMock()}]}
        ctx = api_versions.OpenStackAPIVersions(copy.deepcopy(context))

        ctx.setup()

        self.mock_kc.service_catalog.get_endpoints.assert_called_once_with()
        self.mock_kc.services.list.assert_called_once_with()

        self.assertEqual("computev21", ctx.config["nova"]["service_type"])



# Copyright 2014: Kylin Cloud
# All Rights Reserved.
#
#    Licensed under the Apache License, Version 2.0 (the "License"); you may
#    not use this file except in compliance with the License. You may obtain
#    a copy of the License at
#
#         http://www.apache.org/licenses/LICENSE-2.0
#
#    Unless required by applicable law or agreed to in writing, software
#    distributed under the License is distributed on an "AS IS" BASIS, WITHOUT
#    WARRANTIES OR CONDITIONS OF ANY KIND, either express or implied. See the
#    License for the specific language governing permissions and limitations
#    under the License.

from rally import consts
from rally.plugins.openstack import scenario
from rally.plugins.openstack.scenarios.quotas import utils
from rally.task import validation


class Quotas(utils.QuotasScenario):
    """Benchmark scenarios for quotas."""

    @validation.required_services(consts.Service.NOVA)
    @validation.required_openstack(admin=True, users=True)
    @scenario.configure(context={"admin_cleanup": ["nova.quotas"]})
    def nova_update(self, max_quota=1024):
        """Update quotas for Nova.

        :param max_quota: Max value to be updated for quota.
        """
        self._update_quotas("nova", self.context["tenant"]["id"],
                            max_quota)

    @validation.required_services(consts.Service.NOVA)
    @validation.required_openstack(admin=True, users=True)
    @scenario.configure(context={"admin_cleanup": ["nova.quotas"]})
    def nova_update_and_delete(self, max_quota=1024):
        """Update and delete quotas for Nova.

        :param max_quota: Max value to be updated for quota.
        """

        self._update_quotas("nova", self.context["tenant"]["id"],
                            max_quota)
        self._delete_quotas("nova", self.context["tenant"]["id"])

    @validation.required_services(consts.Service.CINDER)
    @validation.required_openstack(admin=True, users=True)
    @scenario.configure(context={"admin_cleanup": ["cinder.quotas"]})
    def cinder_update(self, max_quota=1024):
        """Update quotas for Cinder.

        :param max_quota: Max value to be updated for quota.
        """
        self._update_quotas("cinder", self.context["tenant"]["id"],
                            max_quota)

    @validation.required_services(consts.Service.CINDER)
    @validation.required_openstack(admin=True, users=True)
    @scenario.configure(context={"admin_cleanup": ["cinder.quotas"]})
    def cinder_update_and_delete(self, max_quota=1024):
        """Update and Delete quotas for Cinder.

        :param max_quota: Max value to be updated for quota.
        """
        self._update_quotas("cinder", self.context["tenant"]["id"],
                            max_quota)
        self._delete_quotas("cinder", self.context["tenant"]["id"])

    @validation.required_services(consts.Service.NEUTRON)
    @validation.required_openstack(admin=True, users=True)
    @scenario.configure(context={"admin_cleanup": ["neutron.quota"]})
    def neutron_update(self, max_quota=1024):
        """Update quotas for neutron.

        :param max_quota: Max value to be updated for quota.
        """
        quota_update_fn = self.admin_clients("neutron").update_quota
        self._update_quotas("neutron", self.context["tenant"]["id"],
                            max_quota, quota_update_fn)



# Copyright 2015 Mirantis Inc.
# All Rights Reserved.
#
#    Licensed under the Apache License, Version 2.0 (the "License"); you may
#    not use this file except in compliance with the License. You may obtain
#    a copy of the License at
#
#         http://www.apache.org/licenses/LICENSE-2.0
#
#    Unless required by applicable law or agreed to in writing, software
#    distributed under the License is distributed on an "AS IS" BASIS, WITHOUT
#    WARRANTIES OR CONDITIONS OF ANY KIND, either express or implied. See the
#    License for the specific language governing permissions and limitations
#    under the License.

from oslo_config import cfg

from rally.common.i18n import _
from rally.common import logging
from rally.common import utils
from rally import consts as rally_consts
from rally import exceptions
from rally.plugins.openstack.context.manila import consts
from rally.plugins.openstack.scenarios.manila import utils as manila_utils
from rally.task import context

CONF = cfg.CONF
LOG = logging.getLogger(__name__)

CONTEXT_NAME = consts.SHARE_NETWORKS_CONTEXT_NAME


@context.configure(name=CONTEXT_NAME, order=450)
class ManilaShareNetworks(context.Context):
    """This context creates resources specific for Manila project."""
    CONFIG_SCHEMA = {
        "type": "object",
        "$schema": rally_consts.JSON_SCHEMA,
        "properties": {
            # NOTE(vponomaryov): specifies whether manila should use
            # share networks for share creation or not.
            "use_share_networks": {"type": "boolean"},

            # NOTE(vponomaryov): this context arg will be used only when
            # context arg "use_share_networks" is set to True and context
            # 'existing_users' is not empty, considering usage of existing
            # users.
            # Expected value is dict of lists where tenant Name or ID is key
            # and list of share_network Names or IDs is value. Example:
            # share_networks = [
            #     {"tenant_1_name_or_id": ["share_network_1_name_or_id",
            #                              "share_network_2_name_or_id"]},
            #     {"tenant_2_name_or_id": ["share_network_3_name_or_id"]}
            # ]
            # Also, make sure that all 'existing users' in appropriate
            # registered deployment have share networks if its usage is
            # enabled, else Rally will randomly take users that does not
            # satisfy criteria.
            "share_networks": {"type": "object"},
        },
        "additionalProperties": False
    }
    DEFAULT_CONFIG = {
        "use_share_networks": False,
        "share_networks": {},
    }

    def _setup_for_existing_users(self):
        if (self.config["use_share_networks"] and
                not self.config["share_networks"]):
            msg = _("Usage of share networks was enabled but for deployment "
                    "with existing users share networks also should be "
                    "specified via arg 'share_networks'")
            raise exceptions.ContextSetupFailure(
                ctx_name=self.get_name(), msg=msg)

        # Set flag that says we will not delete/cleanup share networks
        self.context[CONTEXT_NAME]["delete_share_networks"] = False

        for tenant_name_or_id, share_networks in self.config[
                "share_networks"].items():
            # Verify project existence
            for tenant in self.context["tenants"].values():
                if tenant_name_or_id in (tenant["id"], tenant["name"]):
                    tenant_id = tenant["id"]
                    existing_user = None
                    for user in self.context["users"]:
                        if user["tenant_id"] == tenant_id:
                            existing_user = user
                            break
                    break
            else:
                msg = _("Provided tenant Name or ID '%s' was not found in "
                        "existing tenants.") % tenant_name_or_id
                raise exceptions.ContextSetupFailure(
                    ctx_name=self.get_name(), msg=msg)
            self.context["tenants"][tenant_id][CONTEXT_NAME] = {}
            self.context["tenants"][tenant_id][CONTEXT_NAME][
                "share_networks"] = []

            manila_scenario = manila_utils.ManilaScenario(
                {"user": existing_user})
            existing_sns = manila_scenario._list_share_networks(
                detailed=False, search_opts={"project_id": tenant_id})

            for sn_name_or_id in share_networks:
                # Verify share network existence
                for sn in existing_sns:
                    if sn_name_or_id in (sn.id, sn.name):
                        break
                else:
                    msg = _("Specified share network '%(sn)s' does not "
                            "exist for tenant '%(tenant_id)s'") % {
                                "sn": sn_name_or_id, "tenant_id": tenant_id}
                    raise exceptions.ContextSetupFailure(
                        ctx_name=self.get_name(), msg=msg)

                # Set share network for project
                self.context["tenants"][tenant_id][CONTEXT_NAME][
                    "share_networks"].append(sn)

            # Add shared integer var per project that will be used as index
            # for list with share networks. It is required for balancing.
            self.context["tenants"][tenant_id][CONTEXT_NAME]["sn_iterator"] = (
                utils.RAMInt())

    @logging.log_task_wrapper(LOG.info, _("Enter context: `%s`")
                              % CONTEXT_NAME)
    def setup(self):
        self.context[CONTEXT_NAME] = {}
        if not self.config["use_share_networks"]:
            return
        elif self.context["config"].get("existing_users"):
            self._setup_for_existing_users()
        else:
            # TODO(vponomaryov): add support of autocreated resources
            pass

    @logging.log_task_wrapper(LOG.info, _("Exit context: `%s`") % CONTEXT_NAME)
    def cleanup(self):
        # TODO(vponomaryov): add cleanup for autocreated resources when appear.
        return
